:- module('M1', []).

:- op(500, yfx, and).
:- op(400, yfx, or).

HELP evaluate_delay_aux(Delay, X)
Delay = ground([_G11520,1])or ground([1,_G11531])or ground([_G11531,_G11520])
X = _L855
HELP evaluate_delay_aux(Delay, X)
Delay = ground([_G11520])or ground([_G11531])or ground([_G11531,_G11520])
X = _L855
HELP evaluate_delay_aux(Delay, X)
Delay = ground([_G11731,1])or ground([1,_G11742])or ground([_G11742,_G11731])
X = _L855
HELP evaluate_delay_aux(Delay, X)
Delay = ground([_G11731])or ground([_G11742])or ground([_G11742,_G11731])
X = _L855
'M1.Append.P3'('M1.Nil.C0', A, A).
'~M1.Append.P3'('M1.Nil.C0', A, A).
HELP evaluate_delay_aux(Delay, X)
Delay = ground([_G12292,1])or ground([1,_G12303])or ground([_G12303,_G12292])
X = _L878
HELP evaluate_delay_aux(Delay, X)
Delay = ground([_G12292])or ground([_G12303])or ground([_G12303,_G12292])
X = _L878
HELP evaluate_delay_aux(Delay, X)
Delay = ground([_G12567,1])or ground([1,_G12578])or ground([_G12578,_G12567])
X = _L878
HELP evaluate_delay_aux(Delay, X)
Delay = ground([_G12567])or ground([_G12578])or ground([_G12578,_G12567])
X = _L878
'M1.Append.P3'('M1.Cons.N2'(A, B), C, 'M1.Cons.N2'(A, D)) :-
	'M1.Append.P3'(B, C, D).
'~M1.Append.P3'('M1.Cons.N2'(A, B), C, 'M1.Cons.N2'(A, D)) :-
	'~M1.Append.P3'(B, C, D).
HELP evaluate_delay_aux(Delay, X)
Delay = ground([_G13791,1])or ground([1,_G13802])or ground([_G13802,_G13791])
X = _L878
HELP evaluate_delay_aux(Delay, X)
Delay = ground([_G13791])or ground([_G13802])or ground([_G13802,_G13791])
X = _L878
HELP evaluate_delay_aux(Delay, X)
Delay = ground([_G14030,1])or ground([1,_G14041])or ground([_G14041,_G14030])
X = _L878
HELP evaluate_delay_aux(Delay, X)
Delay = ground([_G14030])or ground([_G14041])or ground([_G14041,_G14030])
X = _L878
'M1.Append3.P4'(A, B, D, E) :-
	'M1.Append.P3'(A, B, C),
	'M1.Append.P3'(C, D, E).
'~M1.Append3.P4'(A, B, D, E) :-
	'~M1.Append.P3'(A, B, C),
	'~M1.Append.P3'(C, D, E).
