:- module(scripts,[]).   %% converted djd 
:- style_check(-singleton). % added djd
%% :- module('scripts', []).

:- discontiguous 'scripts.delayInKind.p4'/4. %% added djd
:- discontiguous '~scripts.delayInKind.p4'/4. %% added djd
:- discontiguous 'scripts.hasIntensionalSet.p1'/1. %% added djd
:- discontiguous '~scripts.hasIntensionalSet.p1'/1. %% added djd
:- discontiguous 'scripts.openHeadPred.p2.0'/2. %% added djd
:- discontiguous '~scripts.openHeadPred.p2.0'/2. %% added djd
:- discontiguous 'scripts.reformatDelays.p3'/3. %% added djd
:- discontiguous '~scripts.reformatDelays.p3'/3. %% added djd
:- discontiguous 'scripts.symbolInFormula.p2'/2. %% added djd
:- discontiguous '~scripts.symbolInFormula.p2'/2. %% added djd
:- discontiguous 'scripts.symbolInTerm.p2'/2. %% added djd
:- discontiguous '~scripts.symbolInTerm.p2'/2. %% added djd
:- discontiguous 'scripts.validDeclaration.p2'/2. %% added djd
:- discontiguous '~scripts.validDeclaration.p2'/2. %% added djd

:- op(500, yfx, and).
:- op(400, yfx, or).

'scripts.formulaInScript.p4'(A, B, C, D) :-
        user:goedel_freeze(ground([A,B,C]), 'scripts':'scripts.formulaInScript.p4.0'(A,B,C,D)).
'~scripts.formulaInScript.p4'(A, B, C, D) :-
        user:goedel_freeze(ground([A,B,C]), 'scripts':'~scripts.formulaInScript.p4.0'(A,B,C,D)).
'scripts.formulaInScript.p4.0'('scripts.script.f5'(_,_,A,_,_), B, C, D) :-
        'sharedPrograms':'sharedPrograms.formulaInLanguage.p4'(C, A, B, D).
'~scripts.formulaInScript.p4.0'('scripts.script.f5'(_,_,A,_,_), B, C, D) :-
        'sharedPrograms':'~sharedPrograms.formulaInLanguage.p4'(C, A, B, D).
'scripts.controlInScript.p4'(A, B, C, D) :-
        user:goedel_freeze(ground([A]), 'scripts':'scripts.controlInScript.p4.0'(A,B,C,D)).
'~scripts.controlInScript.p4'(A, B, C, D) :-
        user:goedel_freeze(ground([A]), 'scripts':'~scripts.controlInScript.p4.0'(A,B,C,D)).
'scripts.controlInScript.p4.0'('scripts.script.f5'(_,A,_,_,B), 'metaDefs.name.f4'(C,D,'metaDefs.predicate.c0',E), F, G) :-
        'sharedPrograms':'sharedPrograms.avlFind.p3'(B, C, 'progDefs.code.f2'(_,H)),
        'sharedPrograms':'sharedPrograms.avlFind.p3'(H, D, I),
        'sharedPrograms':'sharedPrograms.findPredDef.p3'(E, I, 'progDefs.predDef.f4'(E,_,J,K)),
        user:goedel_freeze(ground([C,A]), ('scripts':'~scripts.closedModule.p2'(C,A),true->L=J;'lists':'lists.append.p3'(J,K,L))),
        'scripts.reformatDelays.p3'(L, F, G).
'~scripts.controlInScript.p4.0'('scripts.script.f5'(_,A,_,_,B), 'metaDefs.name.f4'(C,D,'metaDefs.predicate.c0',E), F, G) :-
        'sharedPrograms':'~sharedPrograms.avlFind.p3'(B, C, 'progDefs.code.f2'(_,H)),
        'sharedPrograms':'~sharedPrograms.avlFind.p3'(H, D, I),
        'sharedPrograms':'~sharedPrograms.findPredDef.p3'(E, I, 'progDefs.predDef.f4'(E,_,J,K)),
        user:goedel_freeze(ground([C,A]), ('scripts':'~scripts.closedModule.p2'(C,A),true->L=J;'lists':'~lists.append.p3'(J,K,L))),
        '~scripts.reformatDelays.p3'(L, F, G).
'scripts.constantInScript.p3'(A, B, C) :-
        user:goedel_freeze(ground([A]), 'scripts':'scripts.constantInScript.p3.0'(A,B,C)).
'~scripts.constantInScript.p3'(A, B, C) :-
        user:goedel_freeze(ground([A]), 'scripts':'~scripts.constantInScript.p3.0'(A,B,C)).
'scripts.constantInScript.p3.0'('scripts.script.f5'(_,A,B,_,_), C, D) :-
        'scripts.symbolInScript.p4'(C, B, A, 'progDefs.constantDecl.f1'(D)).
'~scripts.constantInScript.p3.0'('scripts.script.f5'(_,A,B,_,_), C, D) :-
        '~scripts.symbolInScript.p4'(C, B, A, 'progDefs.constantDecl.f1'(D)).
'scripts.baseInScript.p2'(A, B) :-
        user:goedel_freeze(ground([A]), 'scripts':'scripts.baseInScript.p2.0'(A,B)).
'~scripts.baseInScript.p2'(A, B) :-
        user:goedel_freeze(ground([A]), 'scripts':'~scripts.baseInScript.p2.0'(A,B)).
'scripts.baseInScript.p2.0'('scripts.script.f5'(_,A,B,_,_), C) :-
        'scripts.typeSymbolInScriptLanguage.p4'(C, B, A, 'progDefs.baseDecl.c0').
'~scripts.baseInScript.p2.0'('scripts.script.f5'(_,A,B,_,_), C) :-
        '~scripts.typeSymbolInScriptLanguage.p4'(C, B, A, 'progDefs.baseDecl.c0').
'scripts.closedModule.p2'(A, B) :-
        'avlTrees':'avlTrees.avlSearch.p3'(B, A, 'progDefs.modDef.f4'(C,_,_,_)),
        C='progDefs.closedKind.c0'.
'~scripts.closedModule.p2'(A, B) :-
        'avlTrees':'~avlTrees.avlSearch.p3'(B, A, 'progDefs.modDef.f4'(C,_,_,_)),
        C='progDefs.closedKind.c0'.
'scripts.constructorInScript.p3'(A, B, C) :-
        user:goedel_freeze(ground([A]), 'scripts':'scripts.constructorInScript.p3.0'(A,B,C)).
'~scripts.constructorInScript.p3'(A, B, C) :-
        user:goedel_freeze(ground([A]), 'scripts':'~scripts.constructorInScript.p3.0'(A,B,C)).
'scripts.constructorInScript.p3.0'('scripts.script.f5'(_,A,B,_,_), C, D) :-
        'scripts.typeSymbolInScriptLanguage.p4'(C, B, A, 'progDefs.constructorDecl.f1'(D)).
'~scripts.constructorInScript.p3.0'('scripts.script.f5'(_,A,B,_,_), C, D) :-
        '~scripts.typeSymbolInScriptLanguage.p4'(C, B, A, 'progDefs.constructorDecl.f1'(D)).
'scripts.deleteScriptPredicate.p5'(A, B, C, D, E) :-
        user:goedel_freeze(ground([A]), 'scripts':'scripts.deleteScriptPredicate.p5.0'(A,B,C,D,E)).
'~scripts.deleteScriptPredicate.p5'(A, B, C, D, E) :-
        user:goedel_freeze(ground([A]), 'scripts':'~scripts.deleteScriptPredicate.p5.0'(A,B,C,D,E)).
'scripts.deleteScriptPredicate.p5.0'(A, B, C, D, E) :-
        'scripts.delDeclaration.p4'(E, B, 'progDefs.predicateDecl.f3'(_,C,D), A).
'~scripts.deleteScriptPredicate.p5.0'(A, B, C, D, E) :-
        '~scripts.delDeclaration.p4'(E, B, 'progDefs.predicateDecl.f3'(_,C,D), A).
'scripts.delayInScript.p3'(A, B, C) :-
        user:goedel_freeze(ground([A]), 'scripts':'scripts.delayInScript.p3.0'(A,B,C)).
'~scripts.delayInScript.p3'(A, B, C) :-
        user:goedel_freeze(ground([A]), 'scripts':'~scripts.delayInScript.p3.0'(A,B,C)).
'scripts.delayInScript.p3.0'('scripts.script.f5'(_,A,_,_,B), C, D) :-
        C='metaDefs.atom.f2'('metaDefs.name.f4'(E,F,_,G),_),
        'sharedPrograms':'sharedPrograms.avlFind.p3'(B, E, 'progDefs.code.f2'(_,H)),
        'sharedPrograms':'sharedPrograms.avlFind.p3'(H, F, I),
        'sharedPrograms':'sharedPrograms.findPredDef.p3'(G, I, 'progDefs.predDef.f4'(G,_,J,K)),
        'avlTrees':'avlTrees.avlSearch.p3'(A, E, 'progDefs.modDef.f4'(L,_,_,_)),
        'scripts.delayInKind.p4'(L, J, K, 'progDefs.delay.f2'(C,D)).
'~scripts.delayInScript.p3.0'('scripts.script.f5'(_,A,_,_,B), C, D) :-
        C='metaDefs.atom.f2'('metaDefs.name.f4'(E,F,_,G),_),
        'sharedPrograms':'~sharedPrograms.avlFind.p3'(B, E, 'progDefs.code.f2'(_,H)),
        'sharedPrograms':'~sharedPrograms.avlFind.p3'(H, F, I),
        'sharedPrograms':'~sharedPrograms.findPredDef.p3'(G, I, 'progDefs.predDef.f4'(G,_,J,K)),
        'avlTrees':'~avlTrees.avlSearch.p3'(A, E, 'progDefs.modDef.f4'(L,_,_,_)),
        '~scripts.delayInKind.p4'(L, J, K, 'progDefs.delay.f2'(C,D)).
'scripts.delDeclaration.p4'('scripts.script.f5'(A,B,C,D,E), F, G, 'scripts.script.f5'(A,B,H,D,E)) :-
        'scripts.deleteSymbol.p5'(F, H, B, G, C),
        'scripts.freeSymbol.p2'(F, E).
'~scripts.delDeclaration.p4'('scripts.script.f5'(A,B,C,D,E), F, G, 'scripts.script.f5'(A,B,H,D,E)) :-
        '~scripts.deleteSymbol.p5'(F, H, B, G, C),
        '~scripts.freeSymbol.p2'(F, E).
'scripts.definitionInScript.p3'(A, B, C) :-
        user:goedel_freeze(ground([A]), 'scripts':'scripts.definitionInScript.p3.0'(A,B,C)).
'~scripts.definitionInScript.p3'(A, B, C) :-
        user:goedel_freeze(ground([A]), 'scripts':'~scripts.definitionInScript.p3.0'(A,B,C)).
'scripts.definitionInScript.p3.0'('scripts.script.f5'(_,A,_,_,B), 'metaDefs.name.f4'(C,D,E,F), G) :-
        'sharedPrograms':'sharedPrograms.avlFind.p3'(B, C, 'progDefs.code.f2'(_,H)),
        user:goedel_freeze(ground([C,A]), user:goedel_not('scripts':'~scripts.closedModule.p2'(C,A))),
        'sharedPrograms':'sharedPrograms.avlFind.p3'(H, D, I),
        'sharedPrograms':'sharedPrograms.findPredDef.p3'(F, I, 'progDefs.predDef.f4'(F,G,_,_)),
        user:goedel_freeze(ground([F]), (F=0,true->E='metaDefs.proposition.c0';E='metaDefs.predicate.c0')).
'~scripts.definitionInScript.p3.0'('scripts.script.f5'(_,A,_,_,B), 'metaDefs.name.f4'(C,D,E,F), G) :-
        'sharedPrograms':'~sharedPrograms.avlFind.p3'(B, C, 'progDefs.code.f2'(_,H)),
        user:goedel_freeze(ground([C,A]), user:goedel_not('scripts':'~scripts.closedModule.p2'(C,A))),
        'sharedPrograms':'~sharedPrograms.avlFind.p3'(H, D, I),
        'sharedPrograms':'~sharedPrograms.findPredDef.p3'(F, I, 'progDefs.predDef.f4'(F,G,_,_)),
        user:goedel_freeze(ground([F]), (F=0,true->E='metaDefs.proposition.c0';E='metaDefs.predicate.c0')).
'scripts.delayInKind.p4'('progDefs.normalKind.c0', A, B, C) :-
        (   'lists':'lists.member.p2'(C, A)
        ;   'lists':'lists.member.p2'(C, B)
        ).
'~scripts.delayInKind.p4'('progDefs.normalKind.c0', A, B, C) :-
        (   'lists':'~lists.member.p2'(C, A)
        ;   'lists':'~lists.member.p2'(C, B)
        ).
'scripts.delayInKind.p4'('progDefs.closedKind.c0', A, _, B) :-
        'lists':'lists.member.p2'(B, A).
'~scripts.delayInKind.p4'('progDefs.closedKind.c0', A, _, B) :-
        'lists':'~lists.member.p2'(B, A).
'scripts.delayInKind.p4'('progDefs.moduleKind.c0', _, A, B) :-
        'lists':'lists.member.p2'(B, A).
'~scripts.delayInKind.p4'('progDefs.moduleKind.c0', _, A, B) :-
        'lists':'~lists.member.p2'(B, A).
'scripts.deleteDelay.p4'(A, B, C, D) :-
        user:goedel_freeze(ground([A]), 'scripts':'scripts.deleteDelay.p4.0'(A,B,C,D)).
'~scripts.deleteDelay.p4'(A, B, C, D) :-
        user:goedel_freeze(ground([A]), 'scripts':'~scripts.deleteDelay.p4.0'(A,B,C,D)).
'scripts.deleteDelay.p4.0'('scripts.script.f5'(A,B,C,D,E), F, G, 'scripts.script.f5'(A,B,C,D,H)) :-
        'syntax':'syntax.predicateAtom.p3'(F, 'metaDefs.name.f4'(I,J,_,K), _),
        'sharedPrograms':'sharedPrograms.avlFind.p3'(E, I, 'progDefs.code.f2'(L,M)),
        user:goedel_freeze(ground([I,B]), user:goedel_not('scripts':'~scripts.closedModule.p2'(I,B))),
        'sharedPrograms':'sharedPrograms.avlFind.p3'(M, J, N),
        'sharedPrograms':'sharedPrograms.pickPredDef.p4'(K, N, 'progDefs.predDef.f4'(K,O,P,Q), R),
        (   'sharedPrograms':'sharedPrograms.pickOne.p3'('progDefs.delay.f2'(F,G), P, S),
            T=Q
        ;   'sharedPrograms':'sharedPrograms.pickOne.p3'('progDefs.delay.f2'(F,G), Q, T),
            S=P
        ),
        user:goedel_freeze(ground([S,T,O]), ((O=[],S=[],T=[]),true->U=R;U=['progDefs.predDef.f4'(K,O,S,T)|R])),
        user:goedel_freeze(ground([U]), (U=[],true->'avlTrees':'avlTrees.avlDelete.p3'(M,J,V);'avlTrees':'avlTrees.avlUpdate.p5'(M,J,U,V,_))),
        'avlTrees':'avlTrees.avlUpdate.p5'(E, I, 'progDefs.code.f2'(L,V), H, _).
'~scripts.deleteDelay.p4.0'('scripts.script.f5'(A,B,C,D,E), F, G, 'scripts.script.f5'(A,B,C,D,H)) :-
        'syntax':'~syntax.predicateAtom.p3'(F, 'metaDefs.name.f4'(I,J,_,K), _),
        'sharedPrograms':'~sharedPrograms.avlFind.p3'(E, I, 'progDefs.code.f2'(L,M)),
        user:goedel_freeze(ground([I,B]), user:goedel_not('scripts':'~scripts.closedModule.p2'(I,B))),
        'sharedPrograms':'~sharedPrograms.avlFind.p3'(M, J, N),
        'sharedPrograms':'~sharedPrograms.pickPredDef.p4'(K, N, 'progDefs.predDef.f4'(K,O,P,Q), R),
        (   'sharedPrograms':'~sharedPrograms.pickOne.p3'('progDefs.delay.f2'(F,G), P, S),
            T=Q
        ;   'sharedPrograms':'~sharedPrograms.pickOne.p3'('progDefs.delay.f2'(F,G), Q, T),
            S=P
        ),
        user:goedel_freeze(ground([S,T,O]), ((O=[],S=[],T=[]),true->U=R;U=['progDefs.predDef.f4'(K,O,S,T)|R])),
        user:goedel_freeze(ground([U]), (U=[],true->'avlTrees':'~avlTrees.avlDelete.p3'(M,J,V);'avlTrees':'~avlTrees.avlUpdate.p5'(M,J,U,V,_))),
        'avlTrees':'~avlTrees.avlUpdate.p5'(E, I, 'progDefs.code.f2'(L,V), H, _).
'scripts.deleteStatement.p3'(A, B, C) :-
        user:goedel_freeze(ground([A]), 'scripts':'scripts.deleteStatement.p3.0'(A,B,C)).
'~scripts.deleteStatement.p3'(A, B, C) :-
        user:goedel_freeze(ground([A]), 'scripts':'~scripts.deleteStatement.p3.0'(A,B,C)).
'scripts.deleteStatement.p3.0'('scripts.script.f5'(A,B,C,D,E), F, 'scripts.script.f5'(A,B,C,D,G)) :-
        'scripts.openStatementPredicate.p2'(F, 'metaDefs.name.f4'(H,I,_,J)),
        'sharedPrograms':'sharedPrograms.avlFind.p3'(E, H, 'progDefs.code.f2'(K,L)),
        user:goedel_freeze(ground([H,B]), user:goedel_not('scripts':'~scripts.closedModule.p2'(H,B))),
        'sharedPrograms':'sharedPrograms.avlFind.p3'(L, I, M),
        'sharedPrograms':'sharedPrograms.pickPredDef.p4'(J, M, 'progDefs.predDef.f4'(J,N,O,P), Q),
        'lists':'lists.delete.p3'(F, N, R),
        user:goedel_freeze(ground([O,P,R]), ((R=[],O=[],P=[]),true->S=Q;S=['progDefs.predDef.f4'(J,R,O,P)|Q])),
        user:goedel_freeze(ground([S]), (S=[],true->'avlTrees':'avlTrees.avlDelete.p3'(L,I,T);'avlTrees':'avlTrees.avlUpdate.p5'(L,I,S,T,_))),
        'avlTrees':'avlTrees.avlUpdate.p5'(E, H, 'progDefs.code.f2'(K,T), G, _).
'~scripts.deleteStatement.p3.0'('scripts.script.f5'(A,B,C,D,E), F, 'scripts.script.f5'(A,B,C,D,G)) :-
        '~scripts.openStatementPredicate.p2'(F, 'metaDefs.name.f4'(H,I,_,J)),
        'sharedPrograms':'~sharedPrograms.avlFind.p3'(E, H, 'progDefs.code.f2'(K,L)),
        user:goedel_freeze(ground([H,B]), user:goedel_not('scripts':'~scripts.closedModule.p2'(H,B))),
        'sharedPrograms':'~sharedPrograms.avlFind.p3'(L, I, M),
        'sharedPrograms':'~sharedPrograms.pickPredDef.p4'(J, M, 'progDefs.predDef.f4'(J,N,O,P), Q),
        'lists':'~lists.delete.p3'(F, N, R),
        user:goedel_freeze(ground([O,P,R]), ((R=[],O=[],P=[]),true->S=Q;S=['progDefs.predDef.f4'(J,R,O,P)|Q])),
        user:goedel_freeze(ground([S]), (S=[],true->'avlTrees':'~avlTrees.avlDelete.p3'(L,I,T);'avlTrees':'~avlTrees.avlUpdate.p5'(L,I,S,T,_))),
        'avlTrees':'~avlTrees.avlUpdate.p5'(E, H, 'progDefs.code.f2'(K,T), G, _).
'scripts.deleteScriptProposition.p3'(A, B, C) :-
        user:goedel_freeze(ground([A]), 'scripts':'scripts.deleteScriptProposition.p3.0'(A,B,C)).
'~scripts.deleteScriptProposition.p3'(A, B, C) :-
        user:goedel_freeze(ground([A]), 'scripts':'~scripts.deleteScriptProposition.p3.0'(A,B,C)).
'scripts.deleteScriptProposition.p3.0'(A, B, C) :-
        'scripts.delDeclaration.p4'(C, B, 'progDefs.propositionDecl.c0', A).
'~scripts.deleteScriptProposition.p3.0'(A, B, C) :-
        '~scripts.delDeclaration.p4'(C, B, 'progDefs.propositionDecl.c0', A).
'scripts.deleteSymbol.p5'('metaDefs.name.f4'(A,B,C,D), 'progDefs.language.f1'(E), F, G, 'progDefs.language.f1'(H)) :-
        'sharedPrograms':'sharedPrograms.avlFind.p3'(E, A, 'progDefs.module.f3'(I,'progDefs.categories.f2'(J,K),L)),
        user:goedel_freeze(ground([A,F]), user:goedel_not('scripts':'~scripts.closedModule.p2'(A,F))),
        'sharedPrograms':'sharedPrograms.avlFind.p3'(K, B, M),
        'sharedPrograms':'sharedPrograms.findDescriptor.p5'(M, I, C, D, G),
        'lists':'lists.deleteFirst.p3'('progDefs.symbol.f2'(_,G), M, N),
        user:goedel_freeze(ground([N]), (N=[],true->'avlTrees':'avlTrees.avlDelete.p3'(K,B,O);'avlTrees':'avlTrees.avlUpdate.p5'(K,B,N,O,_))),
        'avlTrees':'avlTrees.avlUpdate.p5'(E, A, 'progDefs.module.f3'(I,'progDefs.categories.f2'(J,O),L), H, _).
'~scripts.deleteSymbol.p5'('metaDefs.name.f4'(A,B,C,D), 'progDefs.language.f1'(E), F, G, 'progDefs.language.f1'(H)) :-
        'sharedPrograms':'~sharedPrograms.avlFind.p3'(E, A, 'progDefs.module.f3'(I,'progDefs.categories.f2'(J,K),L)),
        user:goedel_freeze(ground([A,F]), user:goedel_not('scripts':'~scripts.closedModule.p2'(A,F))),
        'sharedPrograms':'~sharedPrograms.avlFind.p3'(K, B, M),
        'sharedPrograms':'~sharedPrograms.findDescriptor.p5'(M, I, C, D, G),
        'lists':'~lists.deleteFirst.p3'('progDefs.symbol.f2'(_,G), M, N),
        user:goedel_freeze(ground([N]), (N=[],true->'avlTrees':'~avlTrees.avlDelete.p3'(K,B,O);'avlTrees':'~avlTrees.avlUpdate.p5'(K,B,N,O,_))),
        'avlTrees':'~avlTrees.avlUpdate.p5'(E, A, 'progDefs.module.f3'(I,'progDefs.categories.f2'(J,O),L), H, _).
'scripts.programToScript.p2'(A, B) :-
        user:goedel_freeze(ground([A]), 'scripts':'scripts.programToScript.p2.0'(A,B)).
'~scripts.programToScript.p2'(A, B) :-
        user:goedel_freeze(ground([A]), 'scripts':'~scripts.programToScript.p2.0'(A,B)).
'scripts.programToScript.p2.0'(A, 'scripts.script.f5'(B,C,D,E,F)) :-
        'programCache':'programCache.unCacheProgram.p2'(A, G),
        'sharedPrograms':'sharedPrograms.moduleLanguage.p3'(G, B, E),
        G='progDefs.program.f4'(B,C,D,F).
'~scripts.programToScript.p2.0'(A, 'scripts.script.f5'(B,C,D,E,F)) :-
        'programCache':'~programCache.unCacheProgram.p2'(A, G),
        'sharedPrograms':'~sharedPrograms.moduleLanguage.p3'(G, B, E),
        G='progDefs.program.f4'(B,C,D,F).
'scripts.insertScriptProposition.p3'(A, B, C) :-
        user:goedel_freeze(ground([A,B]), 'scripts':'scripts.insertScriptProposition.p3.0'(A,B,C)).
'~scripts.insertScriptProposition.p3'(A, B, C) :-
        user:goedel_freeze(ground([A,B]), 'scripts':'~scripts.insertScriptProposition.p3.0'(A,B,C)).
'scripts.insertScriptProposition.p3.0'(A, B, C) :-
        'scripts.insDeclaration.p4'(A, B, 'progDefs.propositionDecl.c0', C).
'~scripts.insertScriptProposition.p3.0'(A, B, C) :-
        '~scripts.insDeclaration.p4'(A, B, 'progDefs.propositionDecl.c0', C).
'scripts.insDeclaration.p4'('scripts.script.f5'(A,B,C,D,E), F, G, 'scripts.script.f5'(A,B,H,D,E)) :-
        'scripts.validDeclaration.p2'(G, C),
        F='metaDefs.name.f4'(I,_,_,_),
        user:goedel_freeze(ground([I,B]), user:goedel_not('scripts':'~scripts.closedModule.p2'(I,B))),
        'sharedPrograms':'sharedPrograms.insertSymbol.p5'(F, C, 'progDefs.exported.c0', G, H).
'~scripts.insDeclaration.p4'('scripts.script.f5'(A,B,C,D,E), F, G, 'scripts.script.f5'(A,B,H,D,E)) :-
        '~scripts.validDeclaration.p2'(G, C),
        F='metaDefs.name.f4'(I,_,_,_),
        user:goedel_freeze(ground([I,B]), user:goedel_not('scripts':'~scripts.closedModule.p2'(I,B))),
        'sharedPrograms':'~sharedPrograms.insertSymbol.p5'(F, C, 'progDefs.exported.c0', G, H).
'scripts.functionInScript.p5'(A, B, C, D, E) :-
        user:goedel_freeze(ground([A]), 'scripts':'scripts.functionInScript.p5.0'(A,B,C,D,E)).
'~scripts.functionInScript.p5'(A, B, C, D, E) :-
        user:goedel_freeze(ground([A]), 'scripts':'~scripts.functionInScript.p5.0'(A,B,C,D,E)).
'scripts.functionInScript.p5.0'('scripts.script.f5'(_,A,B,_,_), C, D, E, F) :-
        'scripts.symbolInScript.p4'(C, B, A, 'progDefs.functionDecl.f4'(_,D,E,F)).
'~scripts.functionInScript.p5.0'('scripts.script.f5'(_,A,B,_,_), C, D, E, F) :-
        '~scripts.symbolInScript.p4'(C, B, A, 'progDefs.functionDecl.f4'(_,D,E,F)).
'scripts.freeSymbol.p2'(A, B) :-
        user:goedel_freeze(ground([B,A]), user:goedel_not(('avlTrees':'~avlTrees.avlMember.p3'(B,_,C),user:goedel_freeze(ground([C,A]),user:goedel_not('scripts':'~scripts.symbolFreeInModuleCode.p2'(A,C)))))).
'~scripts.freeSymbol.p2'(A, B) :-
        user:goedel_freeze(ground([B,A]), user:goedel_not(('avlTrees':'~avlTrees.avlMember.p3'(B,_,C),user:goedel_freeze(ground([C,A]),user:goedel_not('scripts':'~scripts.symbolFreeInModuleCode.p2'(A,C)))))).
'scripts.hasIntensionalSet.p1'('metaDefs.term.f2'(_,A)) :-
        'scripts.hasIntensionalSet1.p1'(A).
'~scripts.hasIntensionalSet.p1'('metaDefs.term.f2'(_,A)) :-
        '~scripts.hasIntensionalSet1.p1'(A).
'scripts.hasIntensionalSet.p1'('metaDefs.suchThat.f2'(_,_)).
'~scripts.hasIntensionalSet.p1'('metaDefs.suchThat.f2'(_,_)).
'scripts.hasIntensionalSet1.p1'([A|B]) :-
        (   'scripts.hasIntensionalSet.p1'(A)
        ;   'scripts.hasIntensionalSet1.p1'(B)
        ).
'~scripts.hasIntensionalSet1.p1'([A|B]) :-
        (   '~scripts.hasIntensionalSet.p1'(A)
        ;   '~scripts.hasIntensionalSet1.p1'(B)
        ).
'scripts.insertDelay.p4'(A, B, C, D) :-
        user:goedel_freeze(ground([A,B,C]), 'scripts':'scripts.insertDelay.p4.0'(A,B,C,D)).
'~scripts.insertDelay.p4'(A, B, C, D) :-
        user:goedel_freeze(ground([A,B,C]), 'scripts':'~scripts.insertDelay.p4.0'(A,B,C,D)).
'scripts.insertDelay.p4.0'('scripts.script.f5'(A,B,C,D,E), F, G, 'scripts.script.f5'(A,B,C,D,H)) :-
        'syntax':'syntax.predicateAtom.p3'(F, 'metaDefs.name.f4'(I,J,_,K), _),
        user:goedel_freeze(ground([I,B]), user:goedel_not('scripts':'~scripts.closedModule.p2'(I,B))),
        'sharedPrograms':'sharedPrograms.formulaInLanguage.p4'(F, C, 'metaDefs.varTyping.f1'([]), _),
        'scripts.intensionalSetFree.p1'(F),
        'avlTrees':'avlTrees.avlIsEmpty.p1'(L),
        'avlTrees':'avlTrees.avlAmend.p6'(E, I, 'progDefs.code.f2'(M,N), 'progDefs.code.f2'(0,L), H, 'progDefs.code.f2'(M,O)),
        'avlTrees':'avlTrees.avlAmend.p6'(O, J, P, [], N, Q),
        user:goedel_freeze(ground([K,Q]), if(('lists':'~lists.deleteFirst.p3'('progDefs.predDef.f4'(K,R,S,T),Q,U),true),P=['progDefs.predDef.f4'(K,R,S,['progDefs.delay.f2'(F,G)|T])|U],P=['progDefs.predDef.f4'(K,[],[],['progDefs.delay.f2'(F,G)])|Q])).
'~scripts.insertDelay.p4.0'('scripts.script.f5'(A,B,C,D,E), F, G, 'scripts.script.f5'(A,B,C,D,H)) :-
        'syntax':'~syntax.predicateAtom.p3'(F, 'metaDefs.name.f4'(I,J,_,K), _),
        user:goedel_freeze(ground([I,B]), user:goedel_not('scripts':'~scripts.closedModule.p2'(I,B))),
        'sharedPrograms':'~sharedPrograms.formulaInLanguage.p4'(F, C, 'metaDefs.varTyping.f1'([]), _),
        '~scripts.intensionalSetFree.p1'(F),
        'avlTrees':'~avlTrees.avlIsEmpty.p1'(L),
        'avlTrees':'~avlTrees.avlAmend.p6'(E, I, 'progDefs.code.f2'(M,N), 'progDefs.code.f2'(0,L), H, 'progDefs.code.f2'(M,O)),
        'avlTrees':'~avlTrees.avlAmend.p6'(O, J, P, [], N, Q),
        user:goedel_freeze(ground([K,Q]), if(('lists':'~lists.deleteFirst.p3'('progDefs.predDef.f4'(K,R,S,T),Q,U),true),P=['progDefs.predDef.f4'(K,R,S,['progDefs.delay.f2'(F,G)|T])|U],P=['progDefs.predDef.f4'(K,[],[],['progDefs.delay.f2'(F,G)])|Q])).
'scripts.insertScriptPredicate.p5'(A, B, C, D, E) :-
        user:goedel_freeze(ground([A,B,C,D]), 'scripts':'scripts.insertScriptPredicate.p5.0'(A,B,C,D,E)).
'~scripts.insertScriptPredicate.p5'(A, B, C, D, E) :-
        user:goedel_freeze(ground([A,B,C,D]), 'scripts':'~scripts.insertScriptPredicate.p5.0'(A,B,C,D,E)).
'scripts.insertScriptPredicate.p5.0'(A, B, C, D, E) :-
        'scripts.insDeclaration.p4'(A, B, 'progDefs.predicateDecl.f3'(_,C,D), E).
'~scripts.insertScriptPredicate.p5.0'(A, B, C, D, E) :-
        '~scripts.insDeclaration.p4'(A, B, 'progDefs.predicateDecl.f3'(_,C,D), E).
'scripts.openHeadPred.p2'(A, B) :-
        user:goedel_freeze(nonvar(A), 'scripts':'scripts.openHeadPred.p2.0'(A,B)).
'~scripts.openHeadPred.p2'(A, B) :-
        user:goedel_freeze(nonvar(A), 'scripts':'~scripts.openHeadPred.p2.0'(A,B)).
'scripts.openHeadPred.p2.0'('metaDefs.pAtom.f1'(A), A).
'~scripts.openHeadPred.p2.0'('metaDefs.pAtom.f1'(A), A).
'scripts.openHeadPred.p2.0'('metaDefs.atom.f2'(A,_), A).
'~scripts.openHeadPred.p2.0'('metaDefs.atom.f2'(A,_), A).
'scripts.insertStatement.p3'(A, B, C) :-
        user:goedel_freeze(ground([A,B]), 'scripts':'scripts.insertStatement.p3.0'(A,B,C)).
'~scripts.insertStatement.p3'(A, B, C) :-
        user:goedel_freeze(ground([A,B]), 'scripts':'~scripts.insertStatement.p3.0'(A,B,C)).
'scripts.insertStatement.p3.0'('scripts.script.f5'(A,B,C,D,E), F, 'scripts.script.f5'(A,B,C,D,G)) :-
        'sharedPrograms':'sharedPrograms.statementInLanguage.p5'(F, C, _, _, []),
        'scripts.openStatementPredicate.p2'(F, 'metaDefs.name.f4'(H,I,_,J)),
        user:goedel_freeze(ground([H,B]), user:goedel_not('scripts':'~scripts.closedModule.p2'(H,B))),
        'avlTrees':'avlTrees.avlIsEmpty.p1'(K),
        'avlTrees':'avlTrees.avlAmend.p6'(E, H, 'progDefs.code.f2'(L,M), 'progDefs.code.f2'(0,K), G, 'progDefs.code.f2'(L,N)),
        'avlTrees':'avlTrees.avlAmend.p6'(N, I, O, [], M, P),
        user:goedel_freeze(ground([J,P]), if(('lists':'~lists.deleteFirst.p3'('progDefs.predDef.f4'(J,Q,R,S),P,T),true),O=['progDefs.predDef.f4'(J,[F|Q],R,S)|T],O=['progDefs.predDef.f4'(J,[F],[],[])|P])).
'~scripts.insertStatement.p3.0'('scripts.script.f5'(A,B,C,D,E), F, 'scripts.script.f5'(A,B,C,D,G)) :-
        'sharedPrograms':'~sharedPrograms.statementInLanguage.p5'(F, C, _, _, []),
        '~scripts.openStatementPredicate.p2'(F, 'metaDefs.name.f4'(H,I,_,J)),
        user:goedel_freeze(ground([H,B]), user:goedel_not('scripts':'~scripts.closedModule.p2'(H,B))),
        'avlTrees':'~avlTrees.avlIsEmpty.p1'(K),
        'avlTrees':'~avlTrees.avlAmend.p6'(E, H, 'progDefs.code.f2'(L,M), 'progDefs.code.f2'(0,K), G, 'progDefs.code.f2'(L,N)),
        'avlTrees':'~avlTrees.avlAmend.p6'(N, I, O, [], M, P),
        user:goedel_freeze(ground([J,P]), if(('lists':'~lists.deleteFirst.p3'('progDefs.predDef.f4'(J,Q,R,S),P,T),true),O=['progDefs.predDef.f4'(J,[F|Q],R,S)|T],O=['progDefs.predDef.f4'(J,[F],[],[])|P])).
'scripts.intensionalSetFree.p1'('metaDefs.atom.f2'(_,A)) :-
        user:goedel_freeze(ground([A]), user:goedel_not('scripts':'~scripts.hasIntensionalSet1.p1'(A))).
'~scripts.intensionalSetFree.p1'('metaDefs.atom.f2'(_,A)) :-
        user:goedel_freeze(ground([A]), user:goedel_not('scripts':'~scripts.hasIntensionalSet1.p1'(A))).
'scripts.predicateInScript.p4'(A, B, C, D) :-
        user:goedel_freeze(ground([A]), 'scripts':'scripts.predicateInScript.p4.0'(A,B,C,D)).
'~scripts.predicateInScript.p4'(A, B, C, D) :-
        user:goedel_freeze(ground([A]), 'scripts':'~scripts.predicateInScript.p4.0'(A,B,C,D)).
'scripts.predicateInScript.p4.0'('scripts.script.f5'(_,A,B,_,_), C, D, E) :-
        'scripts.symbolInScript.p4'(C, B, A, 'progDefs.predicateDecl.f3'(_,D,E)).
'~scripts.predicateInScript.p4.0'('scripts.script.f5'(_,A,B,_,_), C, D, E) :-
        '~scripts.symbolInScript.p4'(C, B, A, 'progDefs.predicateDecl.f3'(_,D,E)).
'scripts.openStatementPredicate.p2'('metaDefs.<-''.f2'(A,_), B) :-
        'scripts.openHeadPred.p2'(A, B).
'~scripts.openStatementPredicate.p2'('metaDefs.<-''.f2'(A,_), B) :-
        '~scripts.openHeadPred.p2'(A, B).
'scripts.symbolInScript.p4'('metaDefs.name.f4'(A,B,C,D), 'progDefs.language.f1'(E), F, G) :-
        'sharedPrograms':'sharedPrograms.avlFind.p3'(E, A, 'progDefs.module.f3'(H,'progDefs.categories.f2'(_,I),_)),
        'avlTrees':'avlTrees.avlSearch.p3'(F, A, 'progDefs.modDef.f4'(J,_,_,_)),
        user:goedel_freeze(ground([J]), (J='progDefs.closedKind.c0',true->H='progDefs.exported.c0';H='progDefs.hidden.c0')),
        'sharedPrograms':'sharedPrograms.avlFind.p3'(I, B, K),
        'sharedPrograms':'sharedPrograms.findDescriptor.p5'(K, H, C, D, G).
'~scripts.symbolInScript.p4'('metaDefs.name.f4'(A,B,C,D), 'progDefs.language.f1'(E), F, G) :-
        'sharedPrograms':'~sharedPrograms.avlFind.p3'(E, A, 'progDefs.module.f3'(H,'progDefs.categories.f2'(_,I),_)),
        'avlTrees':'~avlTrees.avlSearch.p3'(F, A, 'progDefs.modDef.f4'(J,_,_,_)),
        user:goedel_freeze(ground([J]), (J='progDefs.closedKind.c0',true->H='progDefs.exported.c0';H='progDefs.hidden.c0')),
        'sharedPrograms':'~sharedPrograms.avlFind.p3'(I, B, K),
        'sharedPrograms':'~sharedPrograms.findDescriptor.p5'(K, H, C, D, G).
'scripts.symbolFreeInModuleCode.p2'(A, 'progDefs.code.f2'(_,B)) :-
        user:goedel_freeze(ground([A,B]), user:goedel_not((('avlTrees':'~avlTrees.avlMember.p3'(B,_,C),'lists':'~lists.member.p2'('progDefs.predDef.f4'(_,D,E,F),C)),('lists':'~lists.member.p2'(G,D),'scripts':'~scripts.symbolInFormula.p2'(G,A);'lists':'~lists.member.p2'(H,E),'scripts':'~scripts.symbolInDelay.p2'(H,A);'lists':'~lists.member.p2'(I,F),'scripts':'~scripts.symbolInDelay.p2'(I,A))))).
'~scripts.symbolFreeInModuleCode.p2'(A, 'progDefs.code.f2'(_,B)) :-
        user:goedel_freeze(ground([A,B]), user:goedel_not((('avlTrees':'~avlTrees.avlMember.p3'(B,_,C),'lists':'~lists.member.p2'('progDefs.predDef.f4'(_,D,E,F),C)),('lists':'~lists.member.p2'(G,D),'scripts':'~scripts.symbolInFormula.p2'(G,A);'lists':'~lists.member.p2'(H,E),'scripts':'~scripts.symbolInDelay.p2'(H,A);'lists':'~lists.member.p2'(I,F),'scripts':'~scripts.symbolInDelay.p2'(I,A))))).
'scripts.statementInScript.p2'(A, B) :-
        user:goedel_freeze(ground([A]), 'scripts':'scripts.statementInScript.p2.0'(A,B)).
'~scripts.statementInScript.p2'(A, B) :-
        user:goedel_freeze(ground([A]), 'scripts':'~scripts.statementInScript.p2.0'(A,B)).
'scripts.statementInScript.p2.0'(A, B) :-
        'scripts.openStatementPredicate.p2'(B, C),
        'scripts.definitionInScript.p3'(A, C, D),
        'lists':'lists.member.p2'(B, D).
'~scripts.statementInScript.p2.0'(A, B) :-
        '~scripts.openStatementPredicate.p2'(B, C),
        '~scripts.definitionInScript.p3'(A, C, D),
        'lists':'~lists.member.p2'(B, D).
'scripts.propositionInScript.p2'(A, B) :-
        user:goedel_freeze(ground([A]), 'scripts':'scripts.propositionInScript.p2.0'(A,B)).
'~scripts.propositionInScript.p2'(A, B) :-
        user:goedel_freeze(ground([A]), 'scripts':'~scripts.propositionInScript.p2.0'(A,B)).
'scripts.propositionInScript.p2.0'('scripts.script.f5'(_,A,B,_,_), C) :-
        'scripts.symbolInScript.p4'(C, B, A, 'progDefs.propositionDecl.c0').
'~scripts.propositionInScript.p2.0'('scripts.script.f5'(_,A,B,_,_), C) :-
        '~scripts.symbolInScript.p4'(C, B, A, 'progDefs.propositionDecl.c0').
'scripts.reformatDelays.p3'([], [], []).
'~scripts.reformatDelays.p3'([], [], []).
'scripts.reformatDelays.p3'(['progDefs.delay.f2'(B,C)|A], [B|D], [C|E]) :-
        'scripts.reformatDelays.p3'(A, D, E).
'~scripts.reformatDelays.p3'(['progDefs.delay.f2'(B,C)|A], [B|D], [C|E]) :-
        '~scripts.reformatDelays.p3'(A, D, E).
'scripts.statementMatchAtom.p3'(A, B, C) :-
        user:goedel_freeze(ground([A,B]), 'scripts':'scripts.statementMatchAtom.p3.0'(A,B,C)).
'~scripts.statementMatchAtom.p3'(A, B, C) :-
        user:goedel_freeze(ground([A,B]), 'scripts':'~scripts.statementMatchAtom.p3.0'(A,B,C)).
'scripts.statementMatchAtom.p3.0'('scripts.script.f5'(_,A,B,_,C), D, E) :-
        'scripts.openHeadPred.p2'(D, 'metaDefs.name.f4'(F,G,_,H)),
        'sharedPrograms':'sharedPrograms.formulaInLanguage.p4'(D, B, 'metaDefs.varTyping.f1'([]), _),
        user:goedel_freeze(ground([F,A]), user:goedel_not('scripts':'~scripts.closedModule.p2'(F,A))),
        'avlTrees':'avlTrees.avlSearch.p3'(C, F, 'progDefs.code.f2'(_,I)),
        'avlTrees':'avlTrees.avlSearch.p3'(I, G, J),
        'lists':'lists.memberCheck.p2'('progDefs.predDef.f4'(H,K,_,_), J),
        'lists':'lists.member.p2'(E, K).
'~scripts.statementMatchAtom.p3.0'('scripts.script.f5'(_,A,B,_,C), D, E) :-
        '~scripts.openHeadPred.p2'(D, 'metaDefs.name.f4'(F,G,_,H)),
        'sharedPrograms':'~sharedPrograms.formulaInLanguage.p4'(D, B, 'metaDefs.varTyping.f1'([]), _),
        user:goedel_freeze(ground([F,A]), user:goedel_not('scripts':'~scripts.closedModule.p2'(F,A))),
        'avlTrees':'~avlTrees.avlSearch.p3'(C, F, 'progDefs.code.f2'(_,I)),
        'avlTrees':'~avlTrees.avlSearch.p3'(I, G, J),
        'lists':'~lists.memberCheck.p2'('progDefs.predDef.f4'(H,K,_,_), J),
        'lists':'~lists.member.p2'(E, K).
'scripts.symbolInFormula.p2'('metaDefs.pAtom.f1'(A), A).
'~scripts.symbolInFormula.p2'('metaDefs.pAtom.f1'(A), A).
'scripts.symbolInFormula.p2'('metaDefs.atom.f2'(A,B), C) :-
        (   C=A
        ;   'scripts.symbolInTermList.p2'(B, C)
        ).
'~scripts.symbolInFormula.p2'('metaDefs.atom.f2'(A,B), C) :-
        (   C=A
        ;   '~scripts.symbolInTermList.p2'(B, C)
        ).
'scripts.symbolInFormula.p2'('metaDefs.~''.f1'(A), B) :-
        'scripts.symbolInFormula.p2'(A, B).
'~scripts.symbolInFormula.p2'('metaDefs.~''.f1'(A), B) :-
        '~scripts.symbolInFormula.p2'(A, B).
'scripts.symbolInFormula.p2'('metaDefs.&''.f2'(A,B), C) :-
        (   'scripts.symbolInFormula.p2'(A, C)
        ;   'scripts.symbolInFormula.p2'(B, C)
        ).
'~scripts.symbolInFormula.p2'('metaDefs.&''.f2'(A,B), C) :-
        (   '~scripts.symbolInFormula.p2'(A, C)
        ;   '~scripts.symbolInFormula.p2'(B, C)
        ).
'scripts.symbolInFormula.p2'('metaDefs.\\/''.f2'(A,B), C) :-   % added \ djd
        (   'scripts.symbolInFormula.p2'(A, C)
        ;   'scripts.symbolInFormula.p2'(B, C)
        ).
'~scripts.symbolInFormula.p2'('metaDefs.\\/''.f2'(A,B), C) :-   % added \ djd
        (   '~scripts.symbolInFormula.p2'(A, C)
        ;   '~scripts.symbolInFormula.p2'(B, C)
        ).
'scripts.symbolInFormula.p2'('metaDefs.->''.f2'(A,B), C) :-
        (   'scripts.symbolInFormula.p2'(A, C)
        ;   'scripts.symbolInFormula.p2'(B, C)
        ).
'~scripts.symbolInFormula.p2'('metaDefs.->''.f2'(A,B), C) :-
        (   '~scripts.symbolInFormula.p2'(A, C)
        ;   '~scripts.symbolInFormula.p2'(B, C)
        ).
'scripts.symbolInFormula.p2'('metaDefs.<-''.f2'(A,B), C) :-
        (   'scripts.symbolInFormula.p2'(A, C)
        ;   'scripts.symbolInFormula.p2'(B, C)
        ).
'~scripts.symbolInFormula.p2'('metaDefs.<-''.f2'(A,B), C) :-
        (   '~scripts.symbolInFormula.p2'(A, C)
        ;   '~scripts.symbolInFormula.p2'(B, C)
        ).
'scripts.symbolInFormula.p2'('metaDefs.<->''.f2'(A,B), C) :-
        (   'scripts.symbolInFormula.p2'(A, C)
        ;   'scripts.symbolInFormula.p2'(B, C)
        ).
'~scripts.symbolInFormula.p2'('metaDefs.<->''.f2'(A,B), C) :-
        (   '~scripts.symbolInFormula.p2'(A, C)
        ;   '~scripts.symbolInFormula.p2'(B, C)
        ).
'scripts.symbolInFormula.p2'('metaDefs.all.f2'(_,A), B) :-
        'scripts.symbolInFormula.p2'(A, B).
'~scripts.symbolInFormula.p2'('metaDefs.all.f2'(_,A), B) :-
        '~scripts.symbolInFormula.p2'(A, B).
'scripts.symbolInFormula.p2'('metaDefs.some.f2'(_,A), B) :-
        'scripts.symbolInFormula.p2'(A, B).
'~scripts.symbolInFormula.p2'('metaDefs.some.f2'(_,A), B) :-
        '~scripts.symbolInFormula.p2'(A, B).
'scripts.symbolInFormula.p2'('metaDefs.iSTE.f4'(_,A,B,C), D) :-
        (   'scripts.symbolInFormula.p2'(A, D)
        ;   'scripts.symbolInFormula.p2'(B, D)
        ;   'scripts.symbolInFormula.p2'(C, D)
        ).
'~scripts.symbolInFormula.p2'('metaDefs.iSTE.f4'(_,A,B,C), D) :-
        (   '~scripts.symbolInFormula.p2'(A, D)
        ;   '~scripts.symbolInFormula.p2'(B, D)
        ;   '~scripts.symbolInFormula.p2'(C, D)
        ).
'scripts.symbolInFormula.p2'('metaDefs.iTE.f3'(A,B,C), D) :-
        (   'scripts.symbolInFormula.p2'(A, D)
        ;   'scripts.symbolInFormula.p2'(B, D)
        ;   'scripts.symbolInFormula.p2'(C, D)
        ).
'~scripts.symbolInFormula.p2'('metaDefs.iTE.f3'(A,B,C), D) :-
        (   '~scripts.symbolInFormula.p2'(A, D)
        ;   '~scripts.symbolInFormula.p2'(B, D)
        ;   '~scripts.symbolInFormula.p2'(C, D)
        ).
'scripts.symbolInFormula.p2'('metaDefs.iST.f3'(_,A,B), C) :-
        (   'scripts.symbolInFormula.p2'(A, C)
        ;   'scripts.symbolInFormula.p2'(B, C)
        ).
'~scripts.symbolInFormula.p2'('metaDefs.iST.f3'(_,A,B), C) :-
        (   '~scripts.symbolInFormula.p2'(A, C)
        ;   '~scripts.symbolInFormula.p2'(B, C)
        ).
'scripts.symbolInFormula.p2'('metaDefs.iT.f2'(A,B), C) :-
        (   'scripts.symbolInFormula.p2'(A, C)
        ;   'scripts.symbolInFormula.p2'(B, C)
        ).
'~scripts.symbolInFormula.p2'('metaDefs.iT.f2'(A,B), C) :-
        (   '~scripts.symbolInFormula.p2'(A, C)
        ;   '~scripts.symbolInFormula.p2'(B, C)
        ).
'scripts.symbolInDelay.p2'('progDefs.delay.f2'(A,_), B) :-
        'scripts.symbolInFormula.p2'(A, B).
'~scripts.symbolInDelay.p2'('progDefs.delay.f2'(A,_), B) :-
        '~scripts.symbolInFormula.p2'(A, B).
'scripts.termInScript.p5'(A, B, C, D, E) :-
        user:goedel_freeze(ground([A,B,C]), 'scripts':'scripts.termInScript.p5.0'(A,B,C,D,E)).
'~scripts.termInScript.p5'(A, B, C, D, E) :-
        user:goedel_freeze(ground([A,B,C]), 'scripts':'~scripts.termInScript.p5.0'(A,B,C,D,E)).
'scripts.termInScript.p5.0'('scripts.script.f5'(_,_,A,_,_), B, C, D, E) :-
        'sharedPrograms':'sharedPrograms.termInLanguage.p5'(C, A, B, E, D).
'~scripts.termInScript.p5.0'('scripts.script.f5'(_,_,A,_,_), B, C, D, E) :-
        'sharedPrograms':'~sharedPrograms.termInLanguage.p5'(C, A, B, E, D).
'scripts.symbolInTermList.p2'([A|B], C) :-
        (   'scripts.symbolInTerm.p2'(A, C)
        ;   'scripts.symbolInTermList.p2'(B, C)
        ).
'~scripts.symbolInTermList.p2'([A|B], C) :-
        (   '~scripts.symbolInTerm.p2'(A, C)
        ;   '~scripts.symbolInTermList.p2'(B, C)
        ).
'scripts.symbolInTerm.p2'('metaDefs.cTerm.f1'(A), A).
'~scripts.symbolInTerm.p2'('metaDefs.cTerm.f1'(A), A).
'scripts.symbolInTerm.p2'('metaDefs.term.f2'(A,B), C) :-
        (   A=C
        ;   'scripts.symbolInTermList.p2'(B, C)
        ).
'~scripts.symbolInTerm.p2'('metaDefs.term.f2'(A,B), C) :-
        (   A=C
        ;   '~scripts.symbolInTermList.p2'(B, C)
        ).
'scripts.symbolInTerm.p2'('metaDefs.suchThat.f2'(A,B), C) :-
        (   'scripts.symbolInTerm.p2'(A, C)
        ;   'scripts.symbolInFormula.p2'(B, C)
        ).
'~scripts.symbolInTerm.p2'('metaDefs.suchThat.f2'(A,B), C) :-
        (   '~scripts.symbolInTerm.p2'(A, C)
        ;   '~scripts.symbolInFormula.p2'(B, C)
        ).
'scripts.typeSymbolInScriptLanguage.p4'('metaDefs.name.f4'(A,B,C,D), 'progDefs.language.f1'(E), F, G) :-
        'sharedPrograms':'sharedPrograms.avlFind.p3'(E, A, 'progDefs.module.f3'(_,'progDefs.categories.f2'(H,_),_)),
        'avlTrees':'avlTrees.avlSearch.p3'(F, A, 'progDefs.modDef.f4'(I,_,_,_)),
        user:goedel_freeze(ground([I]), (I='progDefs.closedKind.c0',true->J='progDefs.exported.c0';J='progDefs.hidden.c0')),
        'sharedPrograms':'sharedPrograms.avlFind.p3'(H, B, K),
        'sharedPrograms':'sharedPrograms.findDescriptor.p5'(K, J, C, D, G).
'~scripts.typeSymbolInScriptLanguage.p4'('metaDefs.name.f4'(A,B,C,D), 'progDefs.language.f1'(E), F, G) :-
        'sharedPrograms':'~sharedPrograms.avlFind.p3'(E, A, 'progDefs.module.f3'(_,'progDefs.categories.f2'(H,_),_)),
        'avlTrees':'~avlTrees.avlSearch.p3'(F, A, 'progDefs.modDef.f4'(I,_,_,_)),
        user:goedel_freeze(ground([I]), (I='progDefs.closedKind.c0',true->J='progDefs.exported.c0';J='progDefs.hidden.c0')),
        'sharedPrograms':'~sharedPrograms.avlFind.p3'(H, B, K),
        'sharedPrograms':'~sharedPrograms.findDescriptor.p5'(K, J, C, D, G).
'scripts.typeInScript.p2'(A, B) :-
        user:goedel_freeze(ground([A,B]), 'scripts':'scripts.typeInScript.p2.0'(A,B)).
'~scripts.typeInScript.p2'(A, B) :-
        user:goedel_freeze(ground([A,B]), 'scripts':'~scripts.typeInScript.p2.0'(A,B)).
'scripts.typeInScript.p2.0'('scripts.script.f5'(_,_,A,_,_), B) :-
        'sharedPrograms':'sharedPrograms.typeInLanguage.p2'(B, A).
'~scripts.typeInScript.p2.0'('scripts.script.f5'(_,_,A,_,_), B) :-
        'sharedPrograms':'~sharedPrograms.typeInLanguage.p2'(B, A).
'scripts.validDeclaration.p2'('progDefs.propositionDecl.c0', _).
'~scripts.validDeclaration.p2'('progDefs.propositionDecl.c0', _).
'scripts.validDeclaration.p2'('progDefs.predicateDecl.f3'(_,_,A), B) :-
        'sharedPrograms':'sharedPrograms.listTypesInLanguage.p2'(A, B).
'~scripts.validDeclaration.p2'('progDefs.predicateDecl.f3'(_,_,A), B) :-
        'sharedPrograms':'~sharedPrograms.listTypesInLanguage.p2'(A, B).
