system_module('"Syntax','ProgDefs.ModDef.F4'('ProgDefs.NormalKind.C0','.'('"Strings',[]),'.'('"SharedSyntax','.'('"ExtraSyntax',[])),'.'('"MetaDefs','.'('"Substs',[]))),'ProgDefs.Module.F3'('ProgDefs.Hidden.C0','ProgDefs.Categories.F2'('AVLTrees.Node.F5'('AVLTrees.Node.F5'('AVLTrees.Null.C0','"FunctionInd','.'('ProgDefs.Symbol.F2'('ProgDefs.Exported.C0','ProgDefs.BaseDecl.C0'),[]),'AVLTrees.EQ.C0','AVLTrees.Null.C0'),'"NewLabel','.'('ProgDefs.Symbol.F2'('ProgDefs.Hidden.C0','ProgDefs.BaseDecl.C0'),[]),'AVLTrees.EQ.C0','AVLTrees.Node.F5'('AVLTrees.Null.C0','"PredicateInd','.'('ProgDefs.Symbol.F2'('ProgDefs.Exported.C0','ProgDefs.BaseDecl.C0'),[]),'AVLTrees.EQ.C0','AVLTrees.Null.C0')),'AVLTrees.Node.F5'('AVLTrees.Node.F5'('AVLTrees.Node.F5'('AVLTrees.Node.F5'('AVLTrees.Node.F5'('AVLTrees.Node.F5'('AVLTrees.Node.F5'('AVLTrees.Node.F5'('AVLTrees.Null.C0','"All','.'('ProgDefs.Symbol.F2'('ProgDefs.Exported.C0','ProgDefs.PredicateDecl.F3'(3,'Syntax.NoPredInd.C0','.'('MetaDefs.Type.F2'('MetaDefs.Name.F4'('"Lists','"L
system_module('"SharedPrograms','ProgDefs.ModDef.F4'('ProgDefs.ClosedKind.C0','.'('"ProgDefs','.'('"SharedSyntax',[])),'.'('"Substs',[]),[]),'ProgDefs.Module.F3'('ProgDefs.Hidden.C0','ProgDefs.Categories.F2'('AVLTrees.Node.F5'('AVLTrees.Node.F5'('AVLTrees.Node.F5'('AVLTrees.Null.C0','"Associativity','.'('ProgDefs.Symbol.F2'('ProgDefs.Hidden.C0','ProgDefs.BaseDecl.C0'),[]),'AVLTrees.EQ.C0','AVLTrees.Null.C0'),'"NumTypeRecord','.'('ProgDefs.Symbol.F2'('ProgDefs.Exported.C0','ProgDefs.BaseDecl.C0'),[]),'AVLTrees.LH.C0','AVLTrees.Null.C0'),'"Precedence','.'('ProgDefs.Symbol.F2'('ProgDefs.Hidden.C0','ProgDefs.BaseDecl.C0'),[]),'AVLTrees.EQ.C0','AVLTrees.Node.F5'('AVLTrees.Node.F5'('AVLTrees.Null.C0','"TokenClass','.'('ProgDefs.Symbol.F2'('ProgDefs.Hidden.C0','ProgDefs.BaseDecl.C0'),[]),'AVLTrees.EQ.C0','AVLTrees.Null.C0'),'"TypeErrorInfo','.'('ProgDefs.Symbol.F2'('ProgDefs.Exported.C0','ProgDefs.BaseDecl.C0'),[]),'AVLTrees.LH.C0','AVLTrees.Null.C0')),'AVLTrees.Node.F5'('AVLTrees.Node.F5'('AVLTrees.Node.F5'('AVLTre
system_module('"Programs','ProgDefs.ModDef.F4'('ProgDefs.ClosedKind.C0','.'('"Syntax',[]),'.'('"ProgramCache','.'('"SharedPrograms','.'('"Substs',[]))),'.'('"ProgDefs',[])),'ProgDefs.Module.F3'('ProgDefs.Hidden.C0','ProgDefs.Categories.F2'('AVLTrees.Node.F5'('AVLTrees.Null.C0','"ModulePart','.'('ProgDefs.Symbol.F2'('ProgDefs.Exported.C0','ProgDefs.BaseDecl.C0'),[]),'AVLTrees.EQ.C0','AVLTrees.Null.C0'),'AVLTrees.Node.F5'('AVLTrees.Node.F5'('AVLTrees.Node.F5'('AVLTrees.Node.F5'('AVLTrees.Node.F5'('AVLTrees.Node.F5'('AVLTrees.Node.F5'('AVLTrees.Node.F5'('AVLTrees.Null.C0','"AndCondition','.'('ProgDefs.Symbol.F2'('ProgDefs.Exported.C0','ProgDefs.PredicateDecl.F3'(3,'Syntax.NoPredInd.C0','.'('MetaDefs.BType.F1'('MetaDefs.Name.F4'('"ProgDefs','"Condition','MetaDefs.Base.C0',0)),'.'('MetaDefs.BType.F1'('MetaDefs.Name.F4'('"ProgDefs','"Condition','MetaDefs.Base.C0',0)),'.'('MetaDefs.BType.F1'('MetaDefs.Name.F4'('"ProgDefs','"Condition','MetaDefs.Base.C0',0)),[]))))),[]),'AVLTrees.EQ.C0','AVLTrees.Null.C0'),'"Applicab
system_module('"AVLTrees','ProgDefs.ModDef.F4'('ProgDefs.ClosedKind.C0','.'('"Strings',[]),[],[]),'ProgDefs.Module.F3'('ProgDefs.Hidden.C0','ProgDefs.Categories.F2'('AVLTrees.Node.F5'('AVLTrees.Node.F5'('AVLTrees.Null.C0','"AVLTree','.'('ProgDefs.Symbol.F2'('ProgDefs.Exported.C0','ProgDefs.ConstructorDecl.F1'(1)),[]),'AVLTrees.EQ.C0','AVLTrees.Null.C0'),'"Balance','.'('ProgDefs.Symbol.F2'('ProgDefs.Hidden.C0','ProgDefs.BaseDecl.C0'),[]),'AVLTrees.RH.C0','AVLTrees.Node.F5'('AVLTrees.Node.F5'('AVLTrees.Null.C0','"BinaryTree','.'('ProgDefs.Symbol.F2'('ProgDefs.Exported.C0','ProgDefs.ConstructorDecl.F1'(1)),[]),'AVLTrees.EQ.C0','AVLTrees.Null.C0'),'"Hand','.'('ProgDefs.Symbol.F2'('ProgDefs.Hidden.C0','ProgDefs.BaseDecl.C0'),[]),'AVLTrees.EQ.C0','AVLTrees.Node.F5'('AVLTrees.Null.C0','"YesOrNo','.'('ProgDefs.Symbol.F2'('ProgDefs.Hidden.C0','ProgDefs.BaseDecl.C0'),[]),'AVLTrees.EQ.C0','AVLTrees.Null.C0'))),'AVLTrees.Node.F5'('AVLTrees.Node.F5'('AVLTrees.Node.F5'('AVLTrees.Node.F5'('AVLTrees.Node.F5'('AVLTrees.Null.C
system_module('"Floats','ProgDefs.ModDef.F4'('ProgDefs.ClosedKind.C0','.'('"Integers',[]),[],[]),'ProgDefs.Module.F3'('ProgDefs.Hidden.C0','ProgDefs.Categories.F2'('AVLTrees.Node.F5'('AVLTrees.Null.C0','"Float','.'('ProgDefs.Symbol.F2'('ProgDefs.Exported.C0','ProgDefs.BaseDecl.C0'),[]),'AVLTrees.EQ.C0','AVLTrees.Null.C0'),'AVLTrees.Node.F5'('AVLTrees.Node.F5'('AVLTrees.Node.F5'('AVLTrees.Node.F5'('AVLTrees.Node.F5'('AVLTrees.Null.C0','"*','.'('ProgDefs.Symbol.F2'('ProgDefs.Exported.C0','ProgDefs.FunctionDecl.F4'(2,'Syntax.YFX.F1'(520),'.'('MetaDefs.BType.F1'('MetaDefs.Name.F4'('"Floats','"Float','MetaDefs.Base.C0',0)),'.'('MetaDefs.BType.F1'('MetaDefs.Name.F4'('"Floats','"Float','MetaDefs.Base.C0',0)),[])),'MetaDefs.BType.F1'('MetaDefs.Name.F4'('"Floats','"Float','MetaDefs.Base.C0',0)))),[]),'AVLTrees.RH.C0','AVLTrees.Node.F5'('AVLTrees.Null.C0','"+','.'('ProgDefs.Symbol.F2'('ProgDefs.Exported.C0','ProgDefs.FunctionDecl.F4'(2,'Syntax.YFX.F1'(510),'.'('MetaDefs.BType.F1'('MetaDefs.Name.F4'('"Floats','"Float','
system_module('"Flocks','ProgDefs.ModDef.F4'('ProgDefs.ClosedKind.C0','.'('"Units',[]),'.'('"Lists',[]),[]),'ProgDefs.Module.F3'('ProgDefs.Hidden.C0','ProgDefs.Categories.F2'('AVLTrees.Node.F5'('AVLTrees.Null.C0','"Flock','.'('ProgDefs.Symbol.F2'('ProgDefs.Exported.C0','ProgDefs.BaseDecl.C0'),[]),'AVLTrees.EQ.C0','AVLTrees.Null.C0'),'AVLTrees.Node.F5'('AVLTrees.Node.F5'('AVLTrees.Node.F5'('AVLTrees.Null.C0','"DelNthElement','.'('ProgDefs.Symbol.F2'('ProgDefs.Hidden.C0','ProgDefs.PredicateDecl.F3'(4,'Syntax.NoPredInd.C0','.'('MetaDefs.Type.F2'('MetaDefs.Name.F4'('"Lists','"List','MetaDefs.Constructor.C0',1),'.'('MetaDefs.Par.F2'('"x',0),[])),'.'('MetaDefs.BType.F1'('MetaDefs.Name.F4'('"Integers','"Integer','MetaDefs.Base.C0',0)),'.'('MetaDefs.Par.F2'('"x',0),'.'('MetaDefs.Type.F2'('MetaDefs.Name.F4'('"Lists','"List','MetaDefs.Constructor.C0',1),'.'('MetaDefs.Par.F2'('"x',0),[])),[])))))),[]),'AVLTrees.EQ.C0','AVLTrees.Null.C0'),'"DeleteUnit','.'('ProgDefs.Symbol.F2'('ProgDefs.Exported.C0','ProgDefs.PredicateDe
system_module('"FlocksIO','ProgDefs.ModDef.F4'('ProgDefs.ClosedKind.C0','.'('"Flocks','.'('"IO',[])),'.'('"Lists',[]),[]),'ProgDefs.Module.F3'('ProgDefs.Hidden.C0','ProgDefs.Categories.F2'('AVLTrees.Null.C0','AVLTrees.Node.F5'('AVLTrees.Node.F5'('AVLTrees.Null.C0','"FlockCompile','.'('ProgDefs.Symbol.F2'('ProgDefs.Exported.C0','ProgDefs.PredicateDecl.F3'(2,'Syntax.NoPredInd.C0','.'('MetaDefs.BType.F1'('MetaDefs.Name.F4'('"Strings','"String','MetaDefs.Base.C0',0)),'.'('MetaDefs.BType.F1'('MetaDefs.Name.F4'('"Flocks','"Flock','MetaDefs.Base.C0',0)),[])))),[]),'AVLTrees.RH.C0','AVLTrees.Node.F5'('AVLTrees.Null.C0','"FlockDecompile','.'('ProgDefs.Symbol.F2'('ProgDefs.Exported.C0','ProgDefs.PredicateDecl.F3'(2,'Syntax.NoPredInd.C0','.'('MetaDefs.BType.F1'('MetaDefs.Name.F4'('"Flocks','"Flock','MetaDefs.Base.C0',0)),'.'('MetaDefs.BType.F1'('MetaDefs.Name.F4'('"Strings','"String','MetaDefs.Base.C0',0)),[])))),[]),'AVLTrees.EQ.C0','AVLTrees.Null.C0')),'"GetFlock','.'('ProgDefs.Symbol.F2'('ProgDefs.Exported.C0','ProgD
system_module('"IO','ProgDefs.ModDef.F4'('ProgDefs.ClosedKind.C0','.'('"Strings',[]),[],[]),'ProgDefs.Module.F3'('ProgDefs.Hidden.C0','ProgDefs.Categories.F2'('AVLTrees.Node.F5'('AVLTrees.Node.F5'('AVLTrees.Null.C0','"InputStream','.'('ProgDefs.Symbol.F2'('ProgDefs.Exported.C0','ProgDefs.BaseDecl.C0'),[]),'AVLTrees.EQ.C0','AVLTrees.Null.C0'),'"OutputStream','.'('ProgDefs.Symbol.F2'('ProgDefs.Exported.C0','ProgDefs.BaseDecl.C0'),[]),'AVLTrees.EQ.C0','AVLTrees.Node.F5'('AVLTrees.Null.C0','"ResultOfFind','.'('ProgDefs.Symbol.F2'('ProgDefs.Exported.C0','ProgDefs.BaseDecl.C0'),[]),'AVLTrees.EQ.C0','AVLTrees.Null.C0')),'AVLTrees.Node.F5'('AVLTrees.Node.F5'('AVLTrees.Node.F5'('AVLTrees.Node.F5'('AVLTrees.Node.F5'('AVLTrees.Null.C0','"EndInput','.'('ProgDefs.Symbol.F2'('ProgDefs.Exported.C0','ProgDefs.PredicateDecl.F3'(1,'Syntax.NoPredInd.C0','.'('MetaDefs.BType.F1'('MetaDefs.Name.F4'('"IO','"InputStream','MetaDefs.Base.C0',0)),[]))),[]),'AVLTrees.EQ.C0','AVLTrees.Null.C0'),'"EndOutput','.'('ProgDefs.Symbol.F2'('Prog
system_module('"Integers','ProgDefs.ModDef.F4'('ProgDefs.ClosedKind.C0',[],[],[]),'ProgDefs.Module.F3'('ProgDefs.Hidden.C0','ProgDefs.Categories.F2'('AVLTrees.Node.F5'('AVLTrees.Null.C0','"Integer','.'('ProgDefs.Symbol.F2'('ProgDefs.Exported.C0','ProgDefs.BaseDecl.C0'),[]),'AVLTrees.EQ.C0','AVLTrees.Null.C0'),'AVLTrees.Node.F5'('AVLTrees.Node.F5'('AVLTrees.Node.F5'('AVLTrees.Null.C0','"*','.'('ProgDefs.Symbol.F2'('ProgDefs.Exported.C0','ProgDefs.FunctionDecl.F4'(2,'Syntax.YFX.F1'(520),'.'('MetaDefs.BType.F1'('MetaDefs.Name.F4'('"Integers','"Integer','MetaDefs.Base.C0',0)),'.'('MetaDefs.BType.F1'('MetaDefs.Name.F4'('"Integers','"Integer','MetaDefs.Base.C0',0)),[])),'MetaDefs.BType.F1'('MetaDefs.Name.F4'('"Integers','"Integer','MetaDefs.Base.C0',0)))),[]),'AVLTrees.RH.C0','AVLTrees.Node.F5'('AVLTrees.Null.C0','"+','.'('ProgDefs.Symbol.F2'('ProgDefs.Exported.C0','ProgDefs.FunctionDecl.F4'(2,'Syntax.YFX.F1'(510),'.'('MetaDefs.BType.F1'('MetaDefs.Name.F4'('"Integers','"Integer','MetaDefs.Base.C0',0)),'.'('MetaDefs
system_module('"Lists','ProgDefs.ModDef.F4'('ProgDefs.ClosedKind.C0','.'('"Integers',[]),[],[]),'ProgDefs.Module.F3'('ProgDefs.Hidden.C0','ProgDefs.Categories.F2'('AVLTrees.Node.F5'('AVLTrees.Null.C0','"List','.'('ProgDefs.Symbol.F2'('ProgDefs.Exported.C0','ProgDefs.ConstructorDecl.F1'(1)),[]),'AVLTrees.EQ.C0','AVLTrees.Null.C0'),'AVLTrees.Node.F5'('AVLTrees.Node.F5'('AVLTrees.Node.F5'('AVLTrees.Null.C0','"Append','.'('ProgDefs.Symbol.F2'('ProgDefs.Exported.C0','ProgDefs.PredicateDecl.F3'(3,'Syntax.NoPredInd.C0','.'('MetaDefs.Type.F2'('MetaDefs.Name.F4'('"Lists','"List','MetaDefs.Constructor.C0',1),'.'('MetaDefs.Par.F2'('"a',0),[])),'.'('MetaDefs.Type.F2'('MetaDefs.Name.F4'('"Lists','"List','MetaDefs.Constructor.C0',1),'.'('MetaDefs.Par.F2'('"a',0),[])),'.'('MetaDefs.Type.F2'('MetaDefs.Name.F4'('"Lists','"List','MetaDefs.Constructor.C0',1),'.'('MetaDefs.Par.F2'('"a',0),[])),[]))))),[]),'AVLTrees.EQ.C0','AVLTrees.Null.C0'),'"Cons','.'('ProgDefs.Symbol.F2'('ProgDefs.Exported.C0','ProgDefs.FunctionDecl.F4'(2,'Sy
system_module('"Numbers','ProgDefs.ModDef.F4'('ProgDefs.ClosedKind.C0','.'('"Strings','.'('"Floats','.'('"Rationals',[]))),[],[]),'ProgDefs.Module.F3'('ProgDefs.Hidden.C0','ProgDefs.Categories.F2'('AVLTrees.Null.C0','AVLTrees.Node.F5'('AVLTrees.Node.F5'('AVLTrees.Node.F5'('AVLTrees.Null.C0','"FloatString','.'('ProgDefs.Symbol.F2'('ProgDefs.Exported.C0','ProgDefs.PredicateDecl.F3'(2,'Syntax.NoPredInd.C0','.'('MetaDefs.BType.F1'('MetaDefs.Name.F4'('"Floats','"Float','MetaDefs.Base.C0',0)),'.'('MetaDefs.BType.F1'('MetaDefs.Name.F4'('"Strings','"String','MetaDefs.Base.C0',0)),[])))),[]),'AVLTrees.EQ.C0','AVLTrees.Null.C0'),'"FloatToRational','.'('ProgDefs.Symbol.F2'('ProgDefs.Exported.C0','ProgDefs.PredicateDecl.F3'(2,'Syntax.NoPredInd.C0','.'('MetaDefs.BType.F1'('MetaDefs.Name.F4'('"Floats','"Float','MetaDefs.Base.C0',0)),'.'('MetaDefs.BType.F1'('MetaDefs.Name.F4'('"Rationals','"Rational','MetaDefs.Base.C0',0)),[])))),[]),'AVLTrees.LH.C0','AVLTrees.Null.C0'),'"IntegerString','.'('ProgDefs.Symbol.F2'('ProgDefs.Ex
system_module('"NumbersIO','ProgDefs.ModDef.F4'('ProgDefs.ClosedKind.C0','.'('"Numbers','.'('"IO',[])),[],[]),'ProgDefs.Module.F3'('ProgDefs.Hidden.C0','ProgDefs.Categories.F2'('AVLTrees.Node.F5'('AVLTrees.Null.C0','"FileInfo','.'('ProgDefs.Symbol.F2'('ProgDefs.Exported.C0','ProgDefs.BaseDecl.C0'),[]),'AVLTrees.EQ.C0','AVLTrees.Null.C0'),'AVLTrees.Node.F5'('AVLTrees.Node.F5'('AVLTrees.Node.F5'('AVLTrees.Null.C0','"EOF','.'('ProgDefs.Symbol.F2'('ProgDefs.Exported.C0','ProgDefs.ConstantDecl.F1'('MetaDefs.BType.F1'('MetaDefs.Name.F4'('"NumbersIO','"FileInfo','MetaDefs.Base.C0',0)))),[]),'AVLTrees.EQ.C0','AVLTrees.Null.C0'),'"NotEOF','.'('ProgDefs.Symbol.F2'('ProgDefs.Exported.C0','ProgDefs.ConstantDecl.F1'('MetaDefs.BType.F1'('MetaDefs.Name.F4'('"NumbersIO','"FileInfo','MetaDefs.Base.C0',0)))),[]),'AVLTrees.EQ.C0','AVLTrees.Node.F5'('AVLTrees.Null.C0','"ReadFloat','.'('ProgDefs.Symbol.F2'('ProgDefs.Exported.C0','ProgDefs.PredicateDecl.F3'(3,'Syntax.NoPredInd.C0','.'('MetaDefs.BType.F1'('MetaDefs.Name.F4'('"IO','
system_module('"ProgramsIO','ProgDefs.ModDef.F4'('ProgDefs.ClosedKind.C0','.'('"Programs','.'('"IO',[])),'.'('"ProgDefs',[]),[]),'ProgDefs.Module.F3'('ProgDefs.Hidden.C0','ProgDefs.Categories.F2'('AVLTrees.Null.C0','AVLTrees.Node.F5'('AVLTrees.Node.F5'('AVLTrees.Null.C0','"GetProgram','.'('ProgDefs.Symbol.F2'('ProgDefs.Exported.C0','ProgDefs.PredicateDecl.F3'(2,'Syntax.NoPredInd.C0','.'('MetaDefs.BType.F1'('MetaDefs.Name.F4'('"IO','"InputStream','MetaDefs.Base.C0',0)),'.'('MetaDefs.BType.F1'('MetaDefs.Name.F4'('"ProgDefs','"Program','MetaDefs.Base.C0',0)),[])))),[]),'AVLTrees.RH.C0','AVLTrees.Node.F5'('AVLTrees.Null.C0','"GetProgramAux','.'('ProgDefs.Symbol.F2'('ProgDefs.Hidden.C0','ProgDefs.PredicateDecl.F3'(2,'Syntax.NoPredInd.C0','.'('MetaDefs.BType.F1'('MetaDefs.Name.F4'('"IO','"InputStream','MetaDefs.Base.C0',0)),'.'('MetaDefs.BType.F1'('MetaDefs.Name.F4'('"ProgDefs','"Program','MetaDefs.Base.C0',0)),[])))),[]),'AVLTrees.EQ.C0','AVLTrees.Null.C0')),'"ProgramCompile','.'('ProgDefs.Symbol.F2'('ProgDefs.Exp
system_module('"Rationals','ProgDefs.ModDef.F4'('ProgDefs.ClosedKind.C0','.'('"Integers',[]),[],[]),'ProgDefs.Module.F3'('ProgDefs.Hidden.C0','ProgDefs.Categories.F2'('AVLTrees.Node.F5'('AVLTrees.Null.C0','"Rational','.'('ProgDefs.Symbol.F2'('ProgDefs.Exported.C0','ProgDefs.BaseDecl.C0'),[]),'AVLTrees.EQ.C0','AVLTrees.Null.C0'),'AVLTrees.Node.F5'('AVLTrees.Node.F5'('AVLTrees.Node.F5'('AVLTrees.Null.C0','"*','.'('ProgDefs.Symbol.F2'('ProgDefs.Exported.C0','ProgDefs.FunctionDecl.F4'(2,'Syntax.YFX.F1'(520),'.'('MetaDefs.BType.F1'('MetaDefs.Name.F4'('"Rationals','"Rational','MetaDefs.Base.C0',0)),'.'('MetaDefs.BType.F1'('MetaDefs.Name.F4'('"Rationals','"Rational','MetaDefs.Base.C0',0)),[])),'MetaDefs.BType.F1'('MetaDefs.Name.F4'('"Rationals','"Rational','MetaDefs.Base.C0',0)))),[]),'AVLTrees.RH.C0','AVLTrees.Node.F5'('AVLTrees.Null.C0','"+','.'('ProgDefs.Symbol.F2'('ProgDefs.Exported.C0','ProgDefs.FunctionDecl.F4'(2,'Syntax.YFX.F1'(510),'.'('MetaDefs.BType.F1'('MetaDefs.Name.F4'('"Rationals','"Rational','MetaDefs
system_module('"ScriptsIO','ProgDefs.ModDef.F4'('ProgDefs.ClosedKind.C0','.'('"Scripts','.'('"IO',[])),[],[]),'ProgDefs.Module.F3'('ProgDefs.Hidden.C0','ProgDefs.Categories.F2'('AVLTrees.Null.C0','AVLTrees.Node.F5'('AVLTrees.Null.C0','"GetScript','.'('ProgDefs.Symbol.F2'('ProgDefs.Exported.C0','ProgDefs.PredicateDecl.F3'(2,'Syntax.NoPredInd.C0','.'('MetaDefs.BType.F1'('MetaDefs.Name.F4'('"IO','"InputStream','MetaDefs.Base.C0',0)),'.'('MetaDefs.BType.F1'('MetaDefs.Name.F4'('"Scripts','"Script','MetaDefs.Base.C0',0)),[])))),[]),'AVLTrees.RH.C0','AVLTrees.Node.F5'('AVLTrees.Null.C0','"PutScript','.'('ProgDefs.Symbol.F2'('ProgDefs.Exported.C0','ProgDefs.PredicateDecl.F3'(2,'Syntax.NoPredInd.C0','.'('MetaDefs.BType.F1'('MetaDefs.Name.F4'('"IO','"OutputStream','MetaDefs.Base.C0',0)),'.'('MetaDefs.BType.F1'('MetaDefs.Name.F4'('"Scripts','"Script','MetaDefs.Base.C0',0)),[])))),[]),'AVLTrees.EQ.C0','AVLTrees.Null.C0'))),[]),'AVLTrees.Node.F5'('AVLTrees.Null.C0','"GetScript','.'('ProgDefs.PredDef.F4'(2,[],'.'('ProgDefs
system_module('"Sets','ProgDefs.ModDef.F4'('ProgDefs.ClosedKind.C0','.'('"Integers',[]),[],[]),'ProgDefs.Module.F3'('ProgDefs.Hidden.C0','ProgDefs.Categories.F2'('AVLTrees.Node.F5'('AVLTrees.Null.C0','"Set','.'('ProgDefs.Symbol.F2'('ProgDefs.Exported.C0','ProgDefs.ConstructorDecl.F1'(1)),[]),'AVLTrees.EQ.C0','AVLTrees.Null.C0'),'AVLTrees.Node.F5'('AVLTrees.Node.F5'('AVLTrees.Node.F5'('AVLTrees.Null.C0','"*','.'('ProgDefs.Symbol.F2'('ProgDefs.Exported.C0','ProgDefs.FunctionDecl.F4'(2,'Syntax.YFX.F1'(120),'.'('MetaDefs.Type.F2'('MetaDefs.Name.F4'('"Sets','"Set','MetaDefs.Constructor.C0',1),'.'('MetaDefs.Par.F2'('"a',0),[])),'.'('MetaDefs.Type.F2'('MetaDefs.Name.F4'('"Sets','"Set','MetaDefs.Constructor.C0',1),'.'('MetaDefs.Par.F2'('"a',0),[])),[])),'MetaDefs.Type.F2'('MetaDefs.Name.F4'('"Sets','"Set','MetaDefs.Constructor.C0',1),'.'('MetaDefs.Par.F2'('"a',0),[])))),[]),'AVLTrees.EQ.C0','AVLTrees.Null.C0'),'"+','.'('ProgDefs.Symbol.F2'('ProgDefs.Exported.C0','ProgDefs.FunctionDecl.F4'(2,'Syntax.YFX.F1'(110),'.'('
system_module('"Strings','ProgDefs.ModDef.F4'('ProgDefs.ClosedKind.C0','.'('"Lists',[]),[],[]),'ProgDefs.Module.F3'('ProgDefs.Hidden.C0','ProgDefs.Categories.F2'('AVLTrees.Node.F5'('AVLTrees.Null.C0','"String','.'('ProgDefs.Symbol.F2'('ProgDefs.Exported.C0','ProgDefs.BaseDecl.C0'),[]),'AVLTrees.EQ.C0','AVLTrees.Null.C0'),'AVLTrees.Node.F5'('AVLTrees.Node.F5'('AVLTrees.Node.F5'('AVLTrees.Null.C0','"++','.'('ProgDefs.Symbol.F2'('ProgDefs.Exported.C0','ProgDefs.FunctionDecl.F4'(2,'Syntax.YFX.F1'(500),'.'('MetaDefs.BType.F1'('MetaDefs.Name.F4'('"Strings','"String','MetaDefs.Base.C0',0)),'.'('MetaDefs.BType.F1'('MetaDefs.Name.F4'('"Strings','"String','MetaDefs.Base.C0',0)),[])),'MetaDefs.BType.F1'('MetaDefs.Name.F4'('"Strings','"String','MetaDefs.Base.C0',0)))),[]),'AVLTrees.EQ.C0','AVLTrees.Null.C0'),'"<','.'('ProgDefs.Symbol.F2'('ProgDefs.Exported.C0','ProgDefs.PredicateDecl.F3'(2,'Syntax.ZPZ.C0','.'('MetaDefs.BType.F1'('MetaDefs.Name.F4'('"Strings','"String','MetaDefs.Base.C0',0)),'.'('MetaDefs.BType.F1'('MetaD
system_module('"Tables','ProgDefs.ModDef.F4'('ProgDefs.ClosedKind.C0','.'('"Strings',[]),[],[]),'ProgDefs.Module.F3'('ProgDefs.Hidden.C0','ProgDefs.Categories.F2'('AVLTrees.Node.F5'('AVLTrees.Node.F5'('AVLTrees.Null.C0','"Balance','.'('ProgDefs.Symbol.F2'('ProgDefs.Hidden.C0','ProgDefs.BaseDecl.C0'),[]),'AVLTrees.EQ.C0','AVLTrees.Null.C0'),'"Hand','.'('ProgDefs.Symbol.F2'('ProgDefs.Hidden.C0','ProgDefs.BaseDecl.C0'),[]),'AVLTrees.RH.C0','AVLTrees.Node.F5'('AVLTrees.Null.C0','"Table','.'('ProgDefs.Symbol.F2'('ProgDefs.Exported.C0','ProgDefs.ConstructorDecl.F1'(1)),[]),'AVLTrees.RH.C0','AVLTrees.Node.F5'('AVLTrees.Null.C0','"YesOrNo','.'('ProgDefs.Symbol.F2'('ProgDefs.Hidden.C0','ProgDefs.BaseDecl.C0'),[]),'AVLTrees.EQ.C0','AVLTrees.Null.C0'))),'AVLTrees.Node.F5'('AVLTrees.Node.F5'('AVLTrees.Node.F5'('AVLTrees.Node.F5'('AVLTrees.Node.F5'('AVLTrees.Node.F5'('AVLTrees.Null.C0','"Adjust','.'('ProgDefs.Symbol.F2'('ProgDefs.Hidden.C0','ProgDefs.PredicateDecl.F3'(6,'Syntax.NoPredInd.C0','.'('MetaDefs.BType.F1'('MetaD
system_module('"Theories','ProgDefs.ModDef.F4'('ProgDefs.ClosedKind.C0','.'('"Syntax',[]),[],[]),'ProgDefs.Module.F3'('ProgDefs.Hidden.C0','ProgDefs.Categories.F2'('AVLTrees.Node.F5'('AVLTrees.Null.C0','"Theory','.'('ProgDefs.Symbol.F2'('ProgDefs.Exported.C0','ProgDefs.BaseDecl.C0'),[]),'AVLTrees.EQ.C0','AVLTrees.Null.C0'),'AVLTrees.Node.F5'('AVLTrees.Node.F5'('AVLTrees.Node.F5'('AVLTrees.Node.F5'('AVLTrees.Node.F5'('AVLTrees.Node.F5'('AVLTrees.Null.C0','"AxiomInTheory','.'('ProgDefs.Symbol.F2'('ProgDefs.Exported.C0','ProgDefs.PredicateDecl.F3'(2,'Syntax.NoPredInd.C0','.'('MetaDefs.BType.F1'('MetaDefs.Name.F4'('"Theories','"Theory','MetaDefs.Base.C0',0)),'.'('MetaDefs.BType.F1'('MetaDefs.Name.F4'('"MetaDefs','"Formula','MetaDefs.Base.C0',0)),[])))),[]),'AVLTrees.EQ.C0','AVLTrees.Null.C0'),'"BaseInTheory','.'('ProgDefs.Symbol.F2'('ProgDefs.Exported.C0','ProgDefs.PredicateDecl.F3'(3,'Syntax.NoPredInd.C0','.'('MetaDefs.BType.F1'('MetaDefs.Name.F4'('"Theories','"Theory','MetaDefs.Base.C0',0)),'.'('MetaDefs.BType.
system_module('"TheoriesIO','ProgDefs.ModDef.F4'('ProgDefs.ClosedKind.C0','.'('"Theories','.'('"IO',[])),[],[]),'ProgDefs.Module.F3'('ProgDefs.Hidden.C0','ProgDefs.Categories.F2'('AVLTrees.Null.C0','AVLTrees.Node.F5'('AVLTrees.Node.F5'('AVLTrees.Null.C0','"GetTheory','.'('ProgDefs.Symbol.F2'('ProgDefs.Exported.C0','ProgDefs.PredicateDecl.F3'(2,'Syntax.NoPredInd.C0','.'('MetaDefs.BType.F1'('MetaDefs.Name.F4'('"IO','"InputStream','MetaDefs.Base.C0',0)),'.'('MetaDefs.BType.F1'('MetaDefs.Name.F4'('"Theories','"Theory','MetaDefs.Base.C0',0)),[])))),[]),'AVLTrees.EQ.C0','AVLTrees.Null.C0'),'"PutTheory','.'('ProgDefs.Symbol.F2'('ProgDefs.Exported.C0','ProgDefs.PredicateDecl.F3'(2,'Syntax.NoPredInd.C0','.'('MetaDefs.BType.F1'('MetaDefs.Name.F4'('"IO','"OutputStream','MetaDefs.Base.C0',0)),'.'('MetaDefs.BType.F1'('MetaDefs.Name.F4'('"Theories','"Theory','MetaDefs.Base.C0',0)),[])))),[]),'AVLTrees.RH.C0','AVLTrees.Node.F5'('AVLTrees.Null.C0','"TheoryCompile','.'('ProgDefs.Symbol.F2'('ProgDefs.Exported.C0','ProgDefs.Pre
system_module('"Units','ProgDefs.ModDef.F4'('ProgDefs.ClosedKind.C0','.'('"Strings',[]),'.'('"Lists',[]),[]),'ProgDefs.Module.F3'('ProgDefs.Hidden.C0','ProgDefs.Categories.F2'('AVLTrees.Node.F5'('AVLTrees.Null.C0','"Unit','.'('ProgDefs.Symbol.F2'('ProgDefs.Exported.C0','ProgDefs.BaseDecl.C0'),[]),'AVLTrees.EQ.C0','AVLTrees.Null.C0'),'AVLTrees.Node.F5'('AVLTrees.Node.F5'('AVLTrees.Node.F5'('AVLTrees.Null.C0','"StringToUnit','.'('ProgDefs.Symbol.F2'('ProgDefs.Exported.C0','ProgDefs.PredicateDecl.F3'(2,'Syntax.NoPredInd.C0','.'('MetaDefs.BType.F1'('MetaDefs.Name.F4'('"Strings','"String','MetaDefs.Base.C0',0)),'.'('MetaDefs.BType.F1'('MetaDefs.Name.F4'('"Units','"Unit','MetaDefs.Base.C0',0)),[])))),[]),'AVLTrees.EQ.C0','AVLTrees.Null.C0'),'"Unit','.'('ProgDefs.Symbol.F2'('ProgDefs.Hidden.C0','ProgDefs.FunctionDecl.F4'(2,'Syntax.NoFunctInd.C0','.'('MetaDefs.BType.F1'('MetaDefs.Name.F4'('"Strings','"String','MetaDefs.Base.C0',0)),'.'('MetaDefs.Type.F2'('MetaDefs.Name.F4'('"Lists','"List','MetaDefs.Constructor.C0',1
system_module('"Substs','ProgDefs.ModDef.F4'('ProgDefs.ClosedKind.C0','.'('"MetaDefs',[]),[],[]),'ProgDefs.Module.F3'('ProgDefs.Hidden.C0','ProgDefs.Categories.F2'('AVLTrees.Node.F5'('AVLTrees.Node.F5'('AVLTrees.Null.C0','"Heap','.'('ProgDefs.Symbol.F2'('ProgDefs.Exported.C0','ProgDefs.ConstructorDecl.F1'(1)),[]),'AVLTrees.EQ.C0','AVLTrees.Null.C0'),'"Mode','.'('ProgDefs.Symbol.F2'('ProgDefs.Exported.C0','ProgDefs.BaseDecl.C0'),[]),'AVLTrees.RH.C0','AVLTrees.Node.F5'('AVLTrees.Node.F5'('AVLTrees.Null.C0','"Table','.'('ProgDefs.Symbol.F2'('ProgDefs.Hidden.C0','ProgDefs.ConstructorDecl.F1'(1)),[]),'AVLTrees.EQ.C0','AVLTrees.Null.C0'),'"TermSubst','.'('ProgDefs.Symbol.F2'('ProgDefs.Exported.C0','ProgDefs.BaseDecl.C0'),[]),'AVLTrees.EQ.C0','AVLTrees.Node.F5'('AVLTrees.Null.C0','"TypeSubst','.'('ProgDefs.Symbol.F2'('ProgDefs.Exported.C0','ProgDefs.BaseDecl.C0'),[]),'AVLTrees.EQ.C0','AVLTrees.Null.C0'))),'AVLTrees.Node.F5'('AVLTrees.Node.F5'('AVLTrees.Node.F5'('AVLTrees.Node.F5'('AVLTrees.Node.F5'('AVLTrees.Node.F5
system_module('"ExtraSyntax','ProgDefs.ModDef.F4'('ProgDefs.ClosedKind.C0','.'('"MetaDefs',[]),[],[]),'ProgDefs.Module.F3'('ProgDefs.Hidden.C0','ProgDefs.Categories.F2'('AVLTrees.Null.C0','AVLTrees.Node.F5'('AVLTrees.Null.C0','"EBindingInVarTyping','.'('ProgDefs.Symbol.F2'('ProgDefs.Exported.C0','ProgDefs.PredicateDecl.F3'(3,'Syntax.NoPredInd.C0','.'('MetaDefs.Type.F2'('MetaDefs.Name.F4'('"Lists','"List','MetaDefs.Constructor.C0',1),'.'('MetaDefs.BType.F1'('MetaDefs.Name.F4'('"MetaDefs','"VarTypeBinding','MetaDefs.Base.C0',0)),[])),'.'('MetaDefs.BType.F1'('MetaDefs.Name.F4'('"MetaDefs','"Term','MetaDefs.Base.C0',0)),'.'('MetaDefs.BType.F1'('MetaDefs.Name.F4'('"MetaDefs','"Type','MetaDefs.Base.C0',0)),[]))))),[]),'AVLTrees.RH.C0','AVLTrees.Node.F5'('AVLTrees.Null.C0','"EDelBindingInVarTyping','.'('ProgDefs.Symbol.F2'('ProgDefs.Exported.C0','ProgDefs.PredicateDecl.F3'(4,'Syntax.NoPredInd.C0','.'('MetaDefs.Type.F2'('MetaDefs.Name.F4'('"Lists','"List','MetaDefs.Constructor.C0',1),'.'('MetaDefs.BType.F1'('MetaDefs
system_module('"SharedSyntax','ProgDefs.ModDef.F4'('ProgDefs.ClosedKind.C0','.'('"Substs','.'('"MetaDefs',[])),[],[]),'ProgDefs.Module.F3'('ProgDefs.Hidden.C0','ProgDefs.Categories.F2'('AVLTrees.Null.C0','AVLTrees.Node.F5'('AVLTrees.Node.F5'('AVLTrees.Node.F5'('AVLTrees.Node.F5'('AVLTrees.Node.F5'('AVLTrees.Null.C0','"ArgFreeVars','.'('ProgDefs.Symbol.F2'('ProgDefs.Hidden.C0','ProgDefs.PredicateDecl.F3'(2,'Syntax.NoPredInd.C0','.'('MetaDefs.Type.F2'('MetaDefs.Name.F4'('"Lists','"List','MetaDefs.Constructor.C0',1),'.'('MetaDefs.BType.F1'('MetaDefs.Name.F4'('"MetaDefs','"Term','MetaDefs.Base.C0',0)),[])),'.'('MetaDefs.Type.F2'('MetaDefs.Name.F4'('"Lists','"List','MetaDefs.Constructor.C0',1),'.'('MetaDefs.BType.F1'('MetaDefs.Name.F4'('"MetaDefs','"Term','MetaDefs.Base.C0',0)),[])),[])))),[]),'AVLTrees.EQ.C0','AVLTrees.Null.C0'),'"CheckVariantTypes1','.'('ProgDefs.Symbol.F2'('ProgDefs.Hidden.C0','ProgDefs.PredicateDecl.F3'(4,'Syntax.NoPredInd.C0','.'('MetaDefs.BType.F1'('MetaDefs.Name.F4'('"MetaDefs','"Type','Met
system_module('"MetaDefs','ProgDefs.ModDef.F4'('ProgDefs.ClosedKind.C0','.'('"Strings',[]),[],[]),'ProgDefs.Module.F3'('ProgDefs.Hidden.C0','ProgDefs.Categories.F2'('AVLTrees.Node.F5'('AVLTrees.Node.F5'('AVLTrees.Node.F5'('AVLTrees.Node.F5'('AVLTrees.Null.C0','"Binding','.'('ProgDefs.Symbol.F2'('ProgDefs.Exported.C0','ProgDefs.ConstructorDecl.F1'(1)),[]),'AVLTrees.EQ.C0','AVLTrees.Null.C0'),'"Category','.'('ProgDefs.Symbol.F2'('ProgDefs.Exported.C0','ProgDefs.BaseDecl.C0'),[]),'AVLTrees.LH.C0','AVLTrees.Null.C0'),'"Formula','.'('ProgDefs.Symbol.F2'('ProgDefs.Exported.C0','ProgDefs.BaseDecl.C0'),[]),'AVLTrees.LH.C0','AVLTrees.Node.F5'('AVLTrees.Null.C0','"Name','.'('ProgDefs.Symbol.F2'('ProgDefs.Exported.C0','ProgDefs.BaseDecl.C0'),[]),'AVLTrees.EQ.C0','AVLTrees.Null.C0')),'"Term','.'('ProgDefs.Symbol.F2'('ProgDefs.Exported.C0','ProgDefs.BaseDecl.C0'),[]),'AVLTrees.LH.C0','AVLTrees.Node.F5'('AVLTrees.Node.F5'('AVLTrees.Null.C0','"Type','.'('ProgDefs.Symbol.F2'('ProgDefs.Exported.C0','ProgDefs.BaseDecl.C0'),[])
system_module('"ProgramCache','ProgDefs.ModDef.F4'('ProgDefs.ClosedKind.C0','.'('"ProgDefs',[]),[],[]),'ProgDefs.Module.F3'('ProgDefs.Hidden.C0','ProgDefs.Categories.F2'('AVLTrees.Null.C0','AVLTrees.Node.F5'('AVLTrees.Node.F5'('AVLTrees.Null.C0','"GetCachedProgram','.'('ProgDefs.Symbol.F2'('ProgDefs.Exported.C0','ProgDefs.PredicateDecl.F3'(2,'Syntax.NoPredInd.C0','.'('MetaDefs.BType.F1'('MetaDefs.Name.F4'('"Strings','"String','MetaDefs.Base.C0',0)),'.'('MetaDefs.BType.F1'('MetaDefs.Name.F4'('"ProgDefs','"Program','MetaDefs.Base.C0',0)),[])))),[]),'AVLTrees.EQ.C0','AVLTrees.Null.C0'),'"UnCacheProgram','.'('ProgDefs.Symbol.F2'('ProgDefs.Exported.C0','ProgDefs.PredicateDecl.F3'(2,'Syntax.NoPredInd.C0','.'('MetaDefs.BType.F1'('MetaDefs.Name.F4'('"ProgDefs','"Program','MetaDefs.Base.C0',0)),'.'('MetaDefs.BType.F1'('MetaDefs.Name.F4'('"ProgDefs','"Program','MetaDefs.Base.C0',0)),[])))),[]),'AVLTrees.LH.C0','AVLTrees.Null.C0')),[]),'AVLTrees.Node.F5'('AVLTrees.Null.C0','"UnCacheProgram','.'('ProgDefs.PredDef.F4'(2,[
system_module('"ProgDefs','ProgDefs.ModDef.F4'('ProgDefs.ClosedKind.C0','.'('"AVLTrees','.'('"MetaDefs','.'('"Syntax',[]))),[],[]),'ProgDefs.Module.F3'('ProgDefs.Hidden.C0','ProgDefs.Categories.F2'('AVLTrees.Node.F5'('AVLTrees.Node.F5'('AVLTrees.Node.F5'('AVLTrees.Null.C0','"Accessibility','.'('ProgDefs.Symbol.F2'('ProgDefs.Exported.C0','ProgDefs.BaseDecl.C0'),[]),'AVLTrees.EQ.C0','AVLTrees.Null.C0'),'"CategoryTable','.'('ProgDefs.Symbol.F2'('ProgDefs.Exported.C0','ProgDefs.BaseDecl.C0'),[]),'AVLTrees.RH.C0','AVLTrees.Node.F5'('AVLTrees.Node.F5'('AVLTrees.Null.C0','"Condition','.'('ProgDefs.Symbol.F2'('ProgDefs.Exported.C0','ProgDefs.BaseDecl.C0'),[]),'AVLTrees.EQ.C0','AVLTrees.Null.C0'),'"Declaration','.'('ProgDefs.Symbol.F2'('ProgDefs.Exported.C0','ProgDefs.BaseDecl.C0'),[]),'AVLTrees.EQ.C0','AVLTrees.Node.F5'('AVLTrees.Null.C0','"Delay','.'('ProgDefs.Symbol.F2'('ProgDefs.Exported.C0','ProgDefs.BaseDecl.C0'),[]),'AVLTrees.EQ.C0','AVLTrees.Null.C0'))),'"IModulePart','.'('ProgDefs.Symbol.F2'('ProgDefs.Exporte
system_module('"Scripts','ProgDefs.ModDef.F4'('ProgDefs.ClosedKind.C0','.'('"Programs',[]),'.'('"ProgramCache','.'('"SharedPrograms',[])),[]),'ProgDefs.Module.F3'('ProgDefs.Hidden.C0','ProgDefs.Categories.F2'('AVLTrees.Node.F5'('AVLTrees.Null.C0','"Script','.'('ProgDefs.Symbol.F2'('ProgDefs.Exported.C0','ProgDefs.BaseDecl.C0'),[]),'AVLTrees.EQ.C0','AVLTrees.Null.C0'),'AVLTrees.Node.F5'('AVLTrees.Node.F5'('AVLTrees.Node.F5'('AVLTrees.Node.F5'('AVLTrees.Null.C0','"BaseInScript','.'('ProgDefs.Symbol.F2'('ProgDefs.Exported.C0','ProgDefs.PredicateDecl.F3'(2,'Syntax.NoPredInd.C0','.'('MetaDefs.BType.F1'('MetaDefs.Name.F4'('"Scripts','"Script','MetaDefs.Base.C0',0)),'.'('MetaDefs.BType.F1'('MetaDefs.Name.F4'('"MetaDefs','"Name','MetaDefs.Base.C0',0)),[])))),[]),'AVLTrees.RH.C0','AVLTrees.Node.F5'('AVLTrees.Null.C0','"ClosedModule','.'('ProgDefs.Symbol.F2'('ProgDefs.Hidden.C0','ProgDefs.PredicateDecl.F3'(2,'Syntax.NoPredInd.C0','.'('MetaDefs.BType.F1'('MetaDefs.Name.F4'('"Strings','"String','MetaDefs.Base.C0',0)),'.'
%% 0 lines cleaned djd
