echo "Starting" `date` >dat1.out
swipl ~/src/lang/goedel/goedel/v2_0/src/dsa.pl <$1.pl >$1.out.tmp 2>$1.err.tmp
echo "Finished" `date` >dat2.out
cat dat1.out dat2.out $1.out.tmp >$1.out.txt.tmp
# $EDITOR $1.err.tmp $1.out.txt.tmp
$EDITOR $1.err.tmp $1.out.txt.tmp


