echo `date` >dat.out
swipl ~/src/lang/goedel/goedel/v1_4/src/init.pl <$1.pl >$1.out.tmp 2>$1.err.tmp
cat dat.out $1.out.tmp >$1.out.txt.tmp
# $EDITOR $1.err.tmp $1.out.txt.tmp
$EDITOR $1.err.tmp $1.out.txt.tmp


