./add.sh aux
./add.sh avltrees
./add.sh AVLTrees
./add.sh builtin
./add.sh checking
./add.sh compiler
./add.sh constraint
./add.sh delay
./add.sh DJD01
./add.sh djd
./add.sh ExtraSyntax
./add.sh Floats
./add.sh FlocksIO
./add.sh Flocks
./add.sh formula
./add.sh gfreeze
./add.sh Integers
./add.sh IO
./add.sh lib
./add.sh Lists
./add.sh NumbersIO
./add.sh Numbers
./add.sh parser
./add.sh ProgramCache
./add.sh ProgramsIO
./add.sh Programs
./add.sh Rationals
./add.sh ScriptsIO
./add.sh Scripts
./add.sh Sets
./add.sh SharedPrograms
./add.sh SharedSyntax
./add.sh statement
./add.sh Strings
./add.sh Substs
./add.sh Syntax
./add.sh sys_modules
./add.sh system
./add.sh Tables
./add.sh term
./add.sh TheoriesIO
./add.sh Theories
./add.sh tokenizer2
./add.sh tokenizer
./add.sh toplev
./add.sh tracer
./add.sh transform
./add.sh Units
./add.sh utilities
